@@ -39,7 +39,7 @@
 -export_type([snmp/0, snmp_option/0]).
 
 -include_lib("exometer_core/include/exometer.hrl").
--include("log.hrl").
+-include("exometer_log.hrl").
 
 -define(MIB_TEMPLATE, "mibs/EXOMETER-METRICS-MIB.mib").
 -define(MIB_DIR, "tmp/" ++ erlang:atom_to_list(?MODULE)).
