@@ -1,41 +1,27 @@
 //-*-Mode:C++;coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
 // ex: set ft=cpp fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 //
-// BSD LICENSE
-// 
-// Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
-// All rights reserved.
-// 
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are met:
-// 
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above copyright
-//       notice, this list of conditions and the following disclaimer in
-//       the documentation and/or other materials provided with the
-//       distribution.
-//     * All advertising materials mentioning features or use of this
-//       software must display the following acknowledgment:
-//         This product includes software developed by Michael Truog
-//     * The name of the author may not be used to endorse or promote
-//       products derived from this software without specific prior
-//       written permission
-// 
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-// DAMAGE.
+// MIT License
+//
+// Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the "Software"),
+// to deal in the Software without restriction, including without limitation
+// the rights to use, copy, modify, merge, publish, distribute, sublicense,
+// and/or sell copies of the Software, and to permit persons to whom the
+// Software is furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+// DEALINGS IN THE SOFTWARE.
 //
 #include <ei.h>
 #include <vector>
@@ -51,6 +37,7 @@
 #include <sys/socket.h>
 #include <signal.h>
 #include <iostream>
+#include "config.h"
 #include "port.hpp"
 #include "realloc_ptr.hpp"
 #include "copy_ptr.hpp"
@@ -779,6 +766,7 @@ int32_t spawn(char protocol,
               uint64_t group_i,
               char * group_str, uint32_t group_str_len,
               int32_t nice,
+              char * chroot_directory, uint32_t chroot_directory_len,
               char * directory, uint32_t directory_len,
               char * filename, uint32_t /*filename_len*/,
               char * argv, uint32_t argv_len,
@@ -983,18 +971,35 @@ int32_t spawn(char protocol,
             }
         }
 
-        if (owner(user_i, user_str, user_str_len,
-                  group_i, group_str, group_str_len))
+        if (owner_get(user_i, user_str, user_str_len,
+                      group_i, group_str, group_str_len))
             ::_exit(spawn_status::invalid_input);
+        if (chroot_directory_len > 1)
+        {
+#if defined(HAVE_CHROOT)
+            if (::chroot(chroot_directory))
+                ::_exit(spawn_status::invalid_input);
+            if (directory_len == 1)
+            {
+                if (::chdir("/"))
+                    ::_exit(spawn_status::invalid_input);
+            }
+#else
+            assert(chroot_directory);
+            ::_exit(spawn_status::invalid_input);
+#endif
+        }
         if (rlimit(rlimits, rlimits_len))
             ::_exit(spawn_status::invalid_input);
         if (nice != 0)
         {
             errno = 0;
-            ::nice(nice);
+            if (::nice(nice)) {} // ignore invalid -1 result
             if (errno != 0)
                 ::_exit(spawn_status::invalid_input);
         }
+        if (owner_set(user_i, group_i))
+            ::_exit(spawn_status::invalid_input);
         if (directory_len > 1)
         {
             if (::chdir(directory))
