@@ -7,45 +7,31 @@
 %%% Use the configuration information to start CloudI processes.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2016 Michael Truog
-%%% @version 1.5.4 {@date} {@time}
+%%% @copyright 2011-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_configurator).
@@ -72,6 +58,7 @@
          nodes_remove/2,
          logging_set/2,
          logging_file_set/2,
+         logging_stdout_set/2,
          logging_level_set/2,
          logging_syslog_set/2,
          logging_formatters_set/2,
@@ -133,15 +120,9 @@
         try F catch exit:{Reason, _} -> {error, Reason} end).
 
 -ifdef(CLOUDI_CORE_STANDALONE).
--ifdef(ERLANG_OTP_VERSION_16).
--else.
--ifdef(ERLANG_OTP_VERSION_17).
--else.
 -dialyzer({no_match,
            [service_update_external/6]}).
 -endif.
--endif.
--endif.
 
 %%%------------------------------------------------------------------------
 %%% External interface functions
@@ -232,6 +213,11 @@ logging_file_set(FilePath, Timeout) ->
                                 {logging_file_set, FilePath,
                                  timeout_decr(Timeout)}, Timeout)).
 
+logging_stdout_set(Stdout, Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE,
+                                {logging_stdout_set, Stdout,
+                                 timeout_decr(Timeout)}, Timeout)).
+
 logging_level_set(Level, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
                                 {logging_level_set, Level,
@@ -506,6 +492,15 @@ handle_call({logging_file_set, FilePath, _}, _,
             {reply, Error, State}
     end;
 
+handle_call({logging_stdout_set, Stdout, _}, _,
+            #state{configuration = Config} = State) ->
+    #config{logging = LoggingConfig} = Config,
+    ok = cloudi_core_i_logger:stdout_set(Stdout),
+    NewConfig = Config#config{
+                    logging = LoggingConfig#config_logging{
+                        stdout = Stdout}},
+    {reply, ok, State#state{configuration = NewConfig}};
+
 handle_call({logging_level_set, Level, _}, _,
             #state{configuration = Config} = State) ->
     #config{logging = LoggingConfig} = Config,
@@ -622,11 +617,9 @@ configure_service([Service | Services], Configured, Timeout) ->
         {ok, NewService} ->
             configure_service(Services, [NewService | Configured], Timeout);
         {error, Reason} = Error ->
-            ServiceDescription = cloudi_core_i_configuration:
-                                 service_format(Service),
             % wait for logging statements to be logged before crashing
             ?LOG_FATAL_SYNC("configure failed: ~p~n~p",
-                            [Reason, ServiceDescription]),
+                            [Reason, service_format(Service)]),
             Error
     end.
 
@@ -991,6 +984,7 @@ service_start_internal(IndexProcess, Pids,
                            dest_list_deny = DestListDeny,
                            dest_list_allow = DestListAllow,
                            options = #config_service_options{
+                               restart_delay = RestartDelay,
                                scope = Scope} = Options,
                            max_r = MaxR,
                            max_t = MaxT,
@@ -1004,12 +998,9 @@ service_start_internal(IndexProcess, Pids,
                   DestRefresh, DestListDeny,
                   DestListAllow, Options, ID],
                  IndexProcess, CountProcess, 1, Scope,
-                 TimeoutTerm, MaxR, MaxT, ID, Timeout) of
+                 TimeoutTerm, RestartDelay, MaxR, MaxT, ID, Timeout) of
         {ok, P} ->
-            {ID, ServiceConfig} = cloudi_core_i_configuration:
-                                  service_format(Service),
-            ?LOG_INFO("~p -> ~p", [{service_id(ID),
-                                    ServiceConfig}, P]),
+            service_format_log(Service, P),
             service_start_internal(IndexProcess + 1, [P | Pids], Service,
                                    GroupLeader, CountProcess, Timeout);
         {error, Reason} ->
@@ -1038,6 +1029,7 @@ service_start_external(IndexProcess, Pids,
                            dest_list_deny = DestListDeny,
                            dest_list_allow = DestListAllow,
                            options = #config_service_options{
+                               restart_delay = RestartDelay,
                                scope = Scope} = Options,
                            max_r = MaxR,
                            max_t = MaxT,
@@ -1052,12 +1044,9 @@ service_start_external(IndexProcess, Pids,
                   DestRefresh, DestListDeny,
                   DestListAllow, Options, ID],
                  IndexProcess, CountProcess, CountThread, Scope,
-                 TimeoutTerm, MaxR, MaxT, ID, Timeout) of
+                 TimeoutTerm, RestartDelay, MaxR, MaxT, ID, Timeout) of
         {ok, P} ->
-            {ID, ServiceConfig} = cloudi_core_i_configuration:
-                                  service_format(Service),
-            ?LOG_INFO("~p -> ~p", [{service_id(ID),
-                                    ServiceConfig}, P]),
+            service_format_log(Service, P),
             service_start_external(IndexProcess + 1, [P | Pids], Service,
                                    CountThread, CountProcess, Timeout);
         {error, Reason} ->
@@ -1226,6 +1215,13 @@ timeout_decr(Timeout)
     when is_integer(Timeout), Timeout >= ?TIMEOUT_DELTA ->
     Timeout - ?TIMEOUT_DELTA.
 
+service_format_log(Service, Pids) ->
+    ?LOG_INFO("~p ->~n    ~p", [service_format(Service), Pids]).
+
+service_format(Service) ->
+    {ID, ServiceConfig} = cloudi_core_i_configuration:service_format(Service),
+    {service_id(ID), ServiceConfig}.
+
 service_id(ID) ->
     cloudi_x_uuid:uuid_to_string(ID, list_nodash).
 
