@@ -30,7 +30,7 @@
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
 %%% @copyright 2009-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @version 1.7.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_logger).
@@ -62,12 +62,14 @@
 -include_lib("kernel/include/file.hrl").
 
 %% logging macros used only within this module
--define(LOG_INFO_T0(Format, Args, State),
-    log_message_internal_t0(info, ?LINE, ?FUNCTION_NAME, ?FUNCTION_ARITY,
+-define(LOG_T0(Level, Format, Args, State),
+    log_message_internal_t0(Level, ?LINE, ?FUNCTION_NAME, ?FUNCTION_ARITY,
                             Format, Args, State)).
--define(LOG_INFO_T1(Format, Args, State),
-    log_message_internal_t1(info, ?LINE, ?FUNCTION_NAME, ?FUNCTION_ARITY,
+-define(LOG_T1(Level, Format, Args, State),
+    log_message_internal_t1(Level, ?LINE, ?FUNCTION_NAME, ?FUNCTION_ARITY,
                             Format, Args, State)).
+-define(LOG_T0_INFO(Format, Args, State), ?LOG_T0(info, Format, Args, State)).
+-define(LOG_T1_INFO(Format, Args, State), ?LOG_T1(info, Format, Args, State)).
 
 -record(state,
     {
@@ -97,6 +99,12 @@
             :: undefined | #config_logging_formatters{},
         formatters_level
             :: undefined | cloudi_service_api:loglevel(),
+        log_time_offset
+            :: cloudi_service_api:loglevel(),
+        log_time_offset_nanoseconds
+            :: non_neg_integer(),
+        log_time_offset_monitor
+            :: reference(),
         aspects_log_before
             :: list(cloudi_service_api:aspect_log_before()),
         aspects_log_after
@@ -463,6 +471,7 @@ init([#config_logging{file = FilePath,
                       redirect = NodeLogger,
                       syslog = SyslogConfig,
                       formatters = FormattersConfig,
+                      log_time_offset = LogTimeOffset,
                       aspects_log_before = AspectsLogBefore,
                       aspects_log_after = AspectsLogAfter}]) ->
     StdoutPort = stdout_open(Stdout),
@@ -477,6 +486,10 @@ init([#config_logging{file = FilePath,
                main_level = MainLevel,
                formatters = FormattersConfig,
                formatters_level = FormattersLevel,
+               log_time_offset = LogTimeOffset,
+               log_time_offset_nanoseconds = time_offset_nanoseconds(),
+               log_time_offset_monitor = erlang:monitor(time_offset,
+                                                        clock_service),
                aspects_log_before = AspectsLogBefore,
                aspects_log_after = AspectsLogAfter,
                logger_node = node(),
@@ -507,7 +520,7 @@ init([#config_logging{file = FilePath,
                     {stop, Reason}
             end;
         {ok, Binary} ->
-            case ?LOG_INFO_T0("redirecting log output to ~ts",
+            case ?LOG_T0_INFO("redirecting log output to ~ts",
                               [NodeLogger],
                               StateNext#state{file_path = FilePath,
                                               interface_module = Binary,
@@ -632,15 +645,37 @@ handle_cast({Level, Timestamp, Node, Pid,
 handle_cast(Request, State) ->
     {stop, cloudi_string:format("Unknown cast \"~p\"~n", [Request]), State}.
 
+handle_info({'CHANGE', Monitor, time_offset, clock_service, TimeOffset},
+            #state{log_time_offset = LogTimeOffset,
+                   log_time_offset_nanoseconds = ValueOld,
+                   log_time_offset_monitor = Monitor} = State) ->
+    ValueNew = time_offset_to_nanoseconds(TimeOffset),
+    {Sign, Change} = if
+        ValueNew >= ValueOld ->
+            {"+", ValueNew - ValueOld};
+        true ->
+            {"-", ValueOld - ValueNew}
+    end,
+    case ?LOG_T0(LogTimeOffset,
+                 "Erlang time_offset changed ~s~s seconds",
+                 [Sign, nanoseconds_to_seconds_list(Change)],
+                 State#state{log_time_offset_nanoseconds = ValueNew}) of
+        {ok, StateNew} ->
+            {noreply, StateNew};
+        {{error, Reason}, StateNew} ->
+            {stop, Reason, StateNew}
+    end;
 handle_info(Request, State) ->
     {stop, cloudi_string:format("Unknown info \"~p\"~n", [Request]), State}.
 
 terminate(_, #state{fd = Fd,
                     stdout = StdoutPort,
-                    syslog = Syslog}) ->
+                    syslog = Syslog,
+                    log_time_offset_monitor = Monitor}) ->
     _ = (catch file:close(Fd)),
     ok = stdout_close(StdoutPort),
     ok = syslog_close(Syslog),
+    true = erlang:demonitor(Monitor),
     ok.
 
 code_change(_, State, _) ->
@@ -656,6 +691,7 @@ log_config_set(#config_logging{file = FilePath,
                                redirect = NodeLogger,
                                syslog = SyslogConfig,
                                formatters = FormattersConfig,
+                               log_time_offset = LogTimeOffset,
                                aspects_log_before = AspectsLogBefore,
                                aspects_log_after = AspectsLogAfter},
                State) ->
@@ -664,7 +700,8 @@ log_config_set(#config_logging{file = FilePath,
                {Stdout, fun log_config_stdout_set/2},
                {SyslogConfig, fun log_config_syslog_set/2},
                {FormattersConfig, fun log_config_formatters_set/2}],
-              State#state{aspects_log_before = AspectsLogBefore,
+              State#state{log_time_offset = LogTimeOffset,
+                          aspects_log_before = AspectsLogBefore,
                           aspects_log_after = AspectsLogAfter}) of
         {ok, _} = Success ->
             ok = cloudi_core_i_nodes:logging_redirect_set(NodeLogger),
@@ -678,7 +715,7 @@ log_config_main_level_set(MainLevel,
     {ok, State};
 log_config_main_level_set(MainLevelNew,
                           #state{main_level = MainLevelOld} = State) ->
-    case ?LOG_INFO_T0("changing main loglevel from ~s to ~s",
+    case ?LOG_T0_INFO("changing main loglevel from ~s to ~s",
                       [MainLevelOld, MainLevelNew], State) of
         {ok, StateNew} ->
             {ok, StateNew#state{main_level = MainLevelNew}};
@@ -703,7 +740,7 @@ log_config_file_set(FilePathNew,
         FilePathOld =:= undefined ->
             "'undefined'"
     end,
-    case ?LOG_INFO_T0("changing file path from ~s to ~s",
+    case ?LOG_T0_INFO("changing file path from ~s to ~s",
                       [FilePathOldStr, FilePathNewStr], State) of
         {ok, #state{fd = FdOld} = StateNew} ->
             file:close(FdOld),
@@ -751,7 +788,7 @@ log_config_syslog_set(SyslogConfig,
     end,
     if
         SyslogLevelNew /= SyslogLevelOld ->
-            case ?LOG_INFO_T0("changing syslog loglevel from ~s to ~s",
+            case ?LOG_T0_INFO("changing syslog loglevel from ~s to ~s",
                               [SyslogLevelOld, SyslogLevelNew], State) of
                 {ok, StateNew} ->
                     {ok, SwitchF(StateNew)};
@@ -778,7 +815,7 @@ log_config_formatters_set(FormattersConfigNew,
     end,
     if
         FormattersLevelNew /= FormattersLevelOld ->
-            case ?LOG_INFO_T0("changing formatters loglevel from ~s to ~s",
+            case ?LOG_T0_INFO("changing formatters loglevel from ~s to ~s",
                               [FormattersLevelOld, FormattersLevelNew],
                               State) of
                 {ok, StateNew} ->
@@ -805,7 +842,7 @@ log_level_update(#state{main_level = MainLevel,
                     StateNew = State#state{
                         interface_module = Binary,
                         level = LevelNew},
-                    ?LOG_INFO_T1("changed loglevel from ~s to ~s",
+                    ?LOG_T1_INFO("changed loglevel from ~s to ~s",
                                  [LevelOld, LevelNew], StateNew),
                     {ok, StateNew};
                 {error, _} = Error ->
@@ -1081,6 +1118,8 @@ flooding_logger(Timestamp1) ->
             end
     end.
 
+log_message_internal_t0(off, _, _, _, _, _, State) ->
+    {ok, State};
 log_message_internal_t0(LevelCheck, Line, Function, Arity, Format, Args,
                         #state{level = Level,
                                logger_node = Node,
@@ -1269,12 +1308,12 @@ log_redirect(Node, DestinationNew,
         true ->
             Node
     end,
-    case ?LOG_INFO_T0("redirecting log output to ~ts",
+    case ?LOG_T0_INFO("redirecting log output to ~ts",
                       [NodeLogger], State) of
         {ok, StateNext} ->
             case load_interface_module(Level, Mode, DestinationNew) of
                 {ok, Binary} ->
-                    ?LOG_INFO_T1("redirected log output from ~ts to ~ts",
+                    ?LOG_T1_INFO("redirected log output from ~ts to ~ts",
                                  [node(), NodeLogger], StateNext),
                     {ok, StateNext#state{interface_module = Binary,
                                          destination = DestinationNew}};
@@ -1798,6 +1837,18 @@ syslog_close(undefined) ->
 syslog_close(Syslog) when is_pid(Syslog) ->
     cloudi_x_syslog_socket:stop_link(Syslog). % asynchronous stop
 
+-ifdef(ERLANG_OTP_VERSION_20_FEATURES).
+time_offset_nanoseconds() ->
+    erlang:time_offset(nanosecond).
+time_offset_to_nanoseconds(TimeOffset) ->
+    erlang:convert_time_unit(TimeOffset, native, nanosecond).
+-else.
+time_offset_nanoseconds() ->
+    erlang:time_offset(nano_seconds).
+time_offset_to_nanoseconds(TimeOffset) ->
+    erlang:convert_time_unit(TimeOffset, native, nano_seconds).
+-endif.
+
 aspects_log([], _, _, _, _, _, _, _, _, _, _) ->
     ok;
 aspects_log([{M, F} | L], Level, Timestamp, Node, Pid,
@@ -1825,6 +1876,22 @@ aspects_log([F | L], Level, Timestamp, Node, Pid,
                         Module, Line, Function, Arity, MetaData, LogMessage)
     end.
 
+nanoseconds_to_seconds_list(NS) ->
+    L0 = int_to_dec_list(NS, 9, $0),
+    [NanoSeconds8, NanoSeconds7, NanoSeconds6,
+     NanoSeconds5, NanoSeconds4, NanoSeconds3,
+     NanoSeconds2, NanoSeconds1, NanoSeconds0 | L1] = lists:reverse(L0),
+    SecondsFraction = [$.,
+                       NanoSeconds0, NanoSeconds1, NanoSeconds2,
+                       NanoSeconds3, NanoSeconds4, NanoSeconds5,
+                       NanoSeconds6, NanoSeconds7, NanoSeconds8],
+    if
+        L1 == [] ->
+            [$0 | SecondsFraction];
+        true ->
+            lists:reverse(L1) ++ SecondsFraction
+    end.
+
 int_to_dec_list(I) when is_integer(I), I >= 0 ->
     int_to_dec_list([], I).
 
@@ -1843,7 +1910,7 @@ int_to_dec_list(L, I, Count, N, Char)
 int_to_dec_list(L, I, Count, N, Char) ->
     int_to_dec_list([int_to_dec(I rem 10) | L], I div 10, Count + 1, N, Char).
 
-int_to_list_pad(L, 0, _) ->
+int_to_list_pad(L, Count, _) when Count =< 0 ->
     L;
 int_to_list_pad(L, Count, Char) ->
     int_to_list_pad([Char | L], Count - 1, Char).
