@@ -4,48 +4,35 @@
 %%%------------------------------------------------------------------------
 %%% @doc
 %%% ==CloudI Rate-Based Configuration==
-%%% Routines for service request rate-based configuration adjustments.
+%%% Routines for service request and termination
+%%% rate-based configuration adjustments.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2013-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2013-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2013-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2013-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_rate_based_configuration).
@@ -58,6 +45,9 @@
          hibernate_reinit/1,
          hibernate_request/1,
          hibernate_check/1,
+         restart_delay_format/1,
+         restart_delay_validate/1,
+         restart_delay_value/3,
          count_process_dynamic_format/1,
          count_process_dynamic_validate/2,
          count_process_dynamic_init/1,
@@ -76,8 +66,16 @@
 -include("cloudi_logger.hrl").
 -include("cloudi_core_i_constants.hrl").
 
+-define(HIBERNATE_METHOD_DEFAULT, rate_request).
 -define(HIBERNATE_PERIOD_DEFAULT, 5). % seconds
 -define(HIBERNATE_RATE_REQUEST_MIN_DEFAULT, 1). % req/sec
+-define(RESTART_DELAY_METHOD_DEFAULT, exponential).
+-define(RESTART_DELAY_EXPONENTIAL_TIME_MIN_DEFAULT, 1). % milliseconds
+-define(RESTART_DELAY_EXPONENTIAL_TIME_MAX_DEFAULT, 500). % milliseconds
+-define(RESTART_DELAY_LINEAR_TIME_MIN_DEFAULT, 0). % milliseconds
+-define(RESTART_DELAY_LINEAR_TIME_SLOPE_DEFAULT, 50). % milliseconds
+-define(RESTART_DELAY_LINEAR_TIME_MAX_DEFAULT, 250). % milliseconds
+-define(COUNT_PROCESS_DYNAMIC_METHOD_DEFAULT, rate_request).
 -define(COUNT_PROCESS_DYNAMIC_PERIOD_DEFAULT, 5). % seconds
 -define(COUNT_PROCESS_DYNAMIC_RATE_REQUEST_MAX_DEFAULT, 1000). % req/sec
 -define(COUNT_PROCESS_DYNAMIC_RATE_REQUEST_MIN_DEFAULT, 100). % req/sec
@@ -96,6 +94,20 @@
         hibernate = false :: boolean()
     }).
 
+-record(restart_delay,
+    {
+        method = undefined :: undefined | exponential | linear | absolute,
+        time_min = undefined
+            :: undefined |
+               cloudi_service_api:restart_delay_value_milliseconds(),
+        time_max = undefined
+            :: undefined |
+               cloudi_service_api:restart_delay_value_milliseconds(),
+        time_slope = undefined
+            :: undefined |
+               cloudi_service_api:restart_delay_value_milliseconds()
+    }).
+
 -record(count_process_dynamic,
     {
         method = undefined :: undefined | rate_request,
@@ -150,28 +162,28 @@ hibernate_validate(Options) ->
 
 %% called by init/1
 
--spec hibernate_init(Hibernate :: #hibernate{}) ->
+-spec hibernate_init(State :: #hibernate{}) ->
     #hibernate{}.
 
 hibernate_init(#hibernate{method = rate_request,
-                          period = Period} = Hibernate) ->
+                          period = Period} = State) ->
     erlang:send_after(Period * 1000, self(),
                       'cloudi_hibernate_rate'),
-    Hibernate#hibernate{count = 0}.
+    State#hibernate{count = 0}.
 
 %% called by handle_info('cloudi_hibernate_rate', ...)
 
--spec hibernate_reinit(Hibernate :: #hibernate{} | boolean()) ->
+-spec hibernate_reinit(State :: #hibernate{} | boolean()) ->
     {boolean(), #hibernate{} | boolean()}.
 
-hibernate_reinit(Hibernate)
-    when is_boolean(Hibernate) ->
-    {Hibernate, Hibernate};
+hibernate_reinit(State)
+    when is_boolean(State) ->
+    {State, State};
 hibernate_reinit(#hibernate{method = rate_request,
                             period = Period,
                             count = Count,
                             rate_min = RateMin,
-                            hibernate = OldValue} = Hibernate) ->
+                            hibernate = OldValue} = State) ->
     RateCurrent = Count / Period,
     erlang:send_after(Period * 1000, self(),
                       'cloudi_hibernate_rate'),
@@ -192,17 +204,17 @@ hibernate_reinit(#hibernate{method = rate_request,
         true ->
             ok
     end,
-    {Value, Hibernate#hibernate{count = 0,
-                                hibernate = Value}}.
+    {Value, State#hibernate{count = 0,
+                            hibernate = Value}}.
 
 %% called when a service request is handled
 
--spec hibernate_request(Hibernate :: #hibernate{}) ->
+-spec hibernate_request(State :: #hibernate{}) ->
     #hibernate{}.
 
 hibernate_request(#hibernate{method = rate_request,
-                             count = Count} = Hibernate) ->
-    Hibernate#hibernate{count = Count + 1}.
+                             count = Count} = State) ->
+    State#hibernate{count = Count + 1}.
 
 %% called to check if the service pid needs to hibernate
 
@@ -218,6 +230,62 @@ hibernate_check(#hibernate{method = rate_request,
 
 %% convert internal state to the configuration format
 
+-spec restart_delay_format(#restart_delay{} | false) ->
+    list({atom(), any()}) | false.
+
+restart_delay_format(false) ->
+    false;
+restart_delay_format(#restart_delay{method = exponential,
+                                    time_min = TimeMin,
+                                    time_max = TimeMax}) ->
+    [{time_exponential_min, ?LIMIT_FORMAT(TimeMin, 1, ?TIMEOUT_MAX_ERLANG)},
+     {time_exponential_max, ?LIMIT_FORMAT(TimeMax, 1, ?TIMEOUT_MAX_ERLANG)}];
+restart_delay_format(#restart_delay{method = linear,
+                                    time_min = TimeMin,
+                                    time_max = TimeMax,
+                                    time_slope = TimeSlope}) ->
+    [{time_linear_min, ?LIMIT_FORMAT(TimeMin, 0, ?TIMEOUT_MAX_ERLANG)},
+     {time_linear_slope, TimeSlope},
+     {time_linear_max, ?LIMIT_FORMAT(TimeMax, 1, ?TIMEOUT_MAX_ERLANG)}];
+restart_delay_format(#restart_delay{method = absolute,
+                                    time_min = TimeValue,
+                                    time_max = TimeValue}) ->
+    [{time_absolute, ?LIMIT_FORMAT(TimeValue, 1, ?TIMEOUT_MAX_ERLANG)}].
+
+%% convert the configuration format to internal state
+
+-spec restart_delay_validate(list({atom(), any()}) | false) ->
+    {ok, #restart_delay{} | false} |
+    {error, {service_options_restart_delay_invalid, any()}}.
+
+restart_delay_validate(false) ->
+    {ok, false};
+restart_delay_validate(Options) ->
+    restart_delay_validate(Options, #restart_delay{}).
+
+%% provide the value result
+
+-spec restart_delay_value(RestartTimes :: list(non_neg_integer()),
+                          MaxT :: non_neg_integer(),
+                          State :: #restart_delay{} | false) ->
+    false |
+    {NewRestartCount :: non_neg_integer(),
+     NewRestartTimes :: list(non_neg_integer()),
+     Value :: 0..?TIMEOUT_MAX_ERLANG}.
+
+restart_delay_value(_, _, false) ->
+    false;
+restart_delay_value(RestartTimes, MaxT,
+                    #restart_delay{} = State) ->
+    SecondsNow = cloudi_timestamp:seconds(),
+    {NewRestartCount,
+     NewRestartTimes} = cloudi_timestamp:seconds_filter(RestartTimes,
+                                                        SecondsNow, MaxT),
+    Value = restart_delay_value_now(NewRestartCount, State),
+    {NewRestartCount, NewRestartTimes, Value}.
+
+%% convert internal state to the configuration format
+
 -spec count_process_dynamic_format(#count_process_dynamic{} | false) ->
     list({atom(), any()}) | false.
 
@@ -256,22 +324,20 @@ count_process_dynamic_validate(Options, CountProcess)
 
 %% called by init/1
 
--spec count_process_dynamic_init(CountProcessDynamic ::
-                                     #count_process_dynamic{}) ->
+-spec count_process_dynamic_init(State :: #count_process_dynamic{}) ->
     #count_process_dynamic{}.
 
 count_process_dynamic_init(#count_process_dynamic{
                                method = rate_request,
-                               period = Period} = CountProcessDynamic) ->
+                               period = Period} = State) ->
     erlang:send_after(Period * 1000, self(),
                       'cloudi_count_process_dynamic_rate'),
-    CountProcessDynamic#count_process_dynamic{count = 0}.
+    State#count_process_dynamic{count = 0}.
 
 %% called by handle_info('cloudi_count_process_dynamic_rate', ...)
 
 -spec count_process_dynamic_reinit(Dispatcher :: pid(),
-                                   CountProcessDynamic ::
-                                       #count_process_dynamic{}) ->
+                                   State :: #count_process_dynamic{}) ->
     #count_process_dynamic{}.
 
 count_process_dynamic_reinit(Dispatcher,
@@ -282,8 +348,7 @@ count_process_dynamic_reinit(Dispatcher,
                                  rate_max = RateMax,
                                  rate_min = RateMin,
                                  count_process_max = CountProcessMax,
-                                 count_process_min = CountProcessMin} =
-                                     CountProcessDynamic)
+                                 count_process_min = CountProcessMin} = State)
     when is_pid(Dispatcher) ->
     RateCurrent = Count / Period,
     if
@@ -300,18 +365,17 @@ count_process_dynamic_reinit(Dispatcher,
     end,
     erlang:send_after(Period * 1000, self(),
                       'cloudi_count_process_dynamic_rate'),
-    CountProcessDynamic#count_process_dynamic{count = 0}.
+    State#count_process_dynamic{count = 0}.
 
 %% called when a service request is handled
 
--spec count_process_dynamic_request(CountProcessDynamic ::
-                                        #count_process_dynamic{}) ->
+-spec count_process_dynamic_request(State :: #count_process_dynamic{}) ->
     #count_process_dynamic{}.
 
 count_process_dynamic_request(#count_process_dynamic{
                                   method = rate_request,
-                                  count = Count} = CountProcessDynamic) ->
-    CountProcessDynamic#count_process_dynamic{count = Count + 1}.
+                                  count = Count} = State) ->
+    State#count_process_dynamic{count = Count + 1}.
 
 %% update the service pid's count_process
 
@@ -337,25 +401,23 @@ count_process_dynamic_terminate(Pid)
 %% set the service pid termination state
 
 -spec count_process_dynamic_terminate_set(ReceiverPid :: pid(),
-                                          CountProcessDynamic ::
-                                              #count_process_dynamic{}) ->
+                                          State :: #count_process_dynamic{}) ->
     #count_process_dynamic{}.
 
 count_process_dynamic_terminate_set(ReceiverPid,
                                     #count_process_dynamic{
                                         method = rate_request,
-                                        period = Period} =
-                                            CountProcessDynamic)
+                                        period = Period} = State)
     when is_pid(ReceiverPid) ->
     ReceiverPid ! 'cloudi_count_process_dynamic_terminate_check',
     erlang:send_after(Period * 1000, ReceiverPid,
                       'cloudi_count_process_dynamic_terminate_now'),
-    CountProcessDynamic#count_process_dynamic{terminate = true}.
+    State#count_process_dynamic{terminate = true}.
 
 %% check the service pid termination state
 
--spec count_process_dynamic_terminated(CountProcessDynamic ::
-                                           #count_process_dynamic{} | false) ->
+-spec count_process_dynamic_terminated(State :: #count_process_dynamic{} |
+                                                false) ->
     boolean().
 
 count_process_dynamic_terminated(false) ->
@@ -393,25 +455,25 @@ rate_request_validate(Options) ->
 
 %% called by init/1
 
--spec rate_request_init(RateRequest :: #rate_request{}) ->
+-spec rate_request_init(State :: #rate_request{}) ->
     #rate_request{}.
 
-rate_request_init(#rate_request{period = Period} = RateRequest) ->
+rate_request_init(#rate_request{period = Period} = State) ->
     erlang:send_after(Period * 1000, self(),
                       'cloudi_rate_request_max_rate'),
-    RateRequest#rate_request{count = 0,
-                             blocking = false}.
+    State#rate_request{count = 0,
+                       blocking = false}.
 
 %% called by handle_info('cloudi_rate_request_max_rate', ...)
 
--spec rate_request_reinit(RateRequest :: #rate_request{} | undefined) ->
+-spec rate_request_reinit(State :: #rate_request{} | undefined) ->
     #rate_request{}.
 
 rate_request_reinit(undefined) ->
     undefined;
 rate_request_reinit(#rate_request{period = Period,
                                   count = Count,
-                                  blocking = Blocking} = RateRequest) ->
+                                  blocking = Blocking} = State) ->
     erlang:send_after(Period * 1000, self(),
                       'cloudi_rate_request_max_rate'),
     if
@@ -422,28 +484,28 @@ rate_request_reinit(#rate_request{period = Period,
         Blocking =:= false ->
             ok
     end,
-    RateRequest#rate_request{count = 0,
-                             blocking = false}.
+    State#rate_request{count = 0,
+                       blocking = false}.
 
 %% called when a service request is handled
 
--spec rate_request_request(RateRequest :: #rate_request{}) ->
+-spec rate_request_request(State :: #rate_request{}) ->
     {boolean(), #rate_request{}}.
 
-rate_request_request(#rate_request{blocking = true} = RateRequest) ->
-    {false, RateRequest};
+rate_request_request(#rate_request{blocking = true} = State) ->
+    {false, State};
 rate_request_request(#rate_request{period = Period,
                                    count = Count,
-                                   rate_max = RateMax} = RateRequest) ->
+                                   rate_max = RateMax} = State) ->
     NewCount = Count + 1,
     NewBlocking = (NewCount / Period) > RateMax,
-    NewRateRequest = if
+    NewState = if
         NewBlocking =:= true ->
-            RateRequest#rate_request{blocking = true};
+            State#rate_request{blocking = true};
         NewBlocking =:= false ->
-            RateRequest#rate_request{count = NewCount}
+            State#rate_request{count = NewCount}
     end,
-    {not NewBlocking, NewRateRequest}.
+    {not NewBlocking, NewState}.
 
 %%%------------------------------------------------------------------------
 %%% Private functions
@@ -452,10 +514,10 @@ rate_request_request(#rate_request{period = Period,
 hibernate_validate([],
                    #hibernate{method = Method,
                               period = Period,
-                              rate_min = RateMin} = Hibernate) ->
+                              rate_min = RateMin} = State) ->
     NewMethod = if
         Method =:= undefined ->
-            rate_request;
+            ?HIBERNATE_METHOD_DEFAULT;
         Method =/= undefined ->
             Method
     end,
@@ -471,26 +533,224 @@ hibernate_validate([],
         RateMin =/= undefined ->
             RateMin
     end,
-    {ok, Hibernate#hibernate{method = NewMethod,
-                             period = NewPeriod,
-                             rate_min = NewRateMin}};
+    {ok, State#hibernate{method = NewMethod,
+                         period = NewPeriod,
+                         rate_min = NewRateMin}};
 hibernate_validate([{rate_request_min, RateMin} | Options],
-                   #hibernate{method = Method} = Hibernate)
+                   #hibernate{method = Method} = State)
     when ((Method =:= undefined) orelse (Method =:= rate_request)),
          is_number(RateMin), RateMin > 0 ->
     hibernate_validate(Options,
-                       Hibernate#hibernate{method = rate_request,
-                                           rate_min = RateMin});
+                       State#hibernate{method = rate_request,
+                                       rate_min = RateMin});
 hibernate_validate([{period, Period} | Options],
-                   #hibernate{} = Hibernate)
+                   #hibernate{} = State)
     when is_integer(Period), Period > 0,
          Period =< (?TIMEOUT_MAX_ERLANG div 1000) ->
     hibernate_validate(Options,
-                       Hibernate#hibernate{period = Period});
+                       State#hibernate{period = Period});
 hibernate_validate([Invalid | _Options],
-                   _Hibernate) ->
+                   _State) ->
     {error, {service_options_hibernate_invalid, Invalid}}.
 
+restart_delay_validate([],
+                       #restart_delay{method = Method,
+                                      time_min = TimeMin,
+                                      time_max = TimeMax,
+                                      time_slope = TimeSlope} = State) ->
+    NewMethod = if
+        Method =:= undefined ->
+            ?RESTART_DELAY_METHOD_DEFAULT;
+        Method =/= undefined ->
+            Method
+    end,
+    NewTimeMin = if
+        TimeMin =:= undefined ->
+            if
+                NewMethod =:= exponential ->
+                    ?RESTART_DELAY_EXPONENTIAL_TIME_MIN_DEFAULT;
+                NewMethod =:= linear ->
+                    ?RESTART_DELAY_LINEAR_TIME_MIN_DEFAULT
+            end;
+        TimeMin =/= undefined ->
+            TimeMin
+    end,
+    NewTimeMax = if
+        TimeMax =:= undefined ->
+            if
+                NewMethod =:= exponential ->
+                    ?RESTART_DELAY_EXPONENTIAL_TIME_MAX_DEFAULT;
+                NewMethod =:= linear ->
+                    ?RESTART_DELAY_LINEAR_TIME_MAX_DEFAULT
+            end;
+        TimeMax =/= undefined ->
+            TimeMax
+    end,
+    NewTimeSlope = if
+        TimeSlope =:= undefined, NewMethod =:= linear ->
+            ?RESTART_DELAY_LINEAR_TIME_SLOPE_DEFAULT;
+        true ->
+            TimeSlope
+    end,
+    if
+        ((NewMethod =:= exponential) orelse (NewMethod =:= linear)) andalso
+        (NewTimeMin == NewTimeMax) ->
+            {error, {service_options_restart_delay_invalid,
+                     time_absolute}};
+        true ->
+            {ok, State#restart_delay{method = NewMethod,
+                                     time_min = NewTimeMin,
+                                     time_max = NewTimeMax,
+                                     time_slope = NewTimeSlope}}
+    end;
+restart_delay_validate([{time_exponential_min, TimeMin} = Option | Options],
+                       #restart_delay{method = Method,
+                                      time_max = TimeMax} = State)
+    when ((Method =:= undefined) orelse (Method =:= exponential)),
+         (is_integer(TimeMin) andalso
+          (TimeMin >= 1) andalso (TimeMin =< ?TIMEOUT_MAX_ERLANG)) orelse
+         (TimeMin =:= limit_min) orelse (TimeMin =:= limit_max) ->
+    NewTimeMin = ?LIMIT_ASSIGN(TimeMin, 1, ?TIMEOUT_MAX_ERLANG),
+    NewTimeMax = if
+        TimeMax =:= undefined ->
+            erlang:max(NewTimeMin, ?RESTART_DELAY_EXPONENTIAL_TIME_MAX_DEFAULT);
+        TimeMax =/= undefined ->
+            TimeMax
+    end,
+    if
+        NewTimeMin =< NewTimeMax ->
+            restart_delay_validate(Options,
+                                   State#restart_delay{
+                                       method = exponential,
+                                       time_min = NewTimeMin,
+                                       time_max = NewTimeMax});
+        true ->
+            {error, {service_options_restart_delay_invalid, Option}}
+    end;
+restart_delay_validate([{time_exponential_max, TimeMax} = Option | Options],
+                       #restart_delay{method = Method,
+                                      time_min = TimeMin} = State)
+    when ((Method =:= undefined) orelse (Method =:= exponential)),
+         (is_integer(TimeMax) andalso
+          (TimeMax >= 1) andalso (TimeMax =< ?TIMEOUT_MAX_ERLANG)) orelse
+         (TimeMax =:= limit_min) orelse (TimeMax =:= limit_max) ->
+    NewTimeMax = ?LIMIT_ASSIGN(TimeMax, 1, ?TIMEOUT_MAX_ERLANG),
+    NewTimeMin = if
+        TimeMin =:= undefined ->
+            erlang:min(NewTimeMax, ?RESTART_DELAY_EXPONENTIAL_TIME_MIN_DEFAULT);
+        TimeMin =/= undefined ->
+            TimeMin
+    end,
+    if
+        NewTimeMin =< NewTimeMax ->
+            restart_delay_validate(Options,
+                                   State#restart_delay{
+                                       method = exponential,
+                                       time_max = NewTimeMax,
+                                       time_min = NewTimeMin});
+        true ->
+            {error, {service_options_restart_delay_invalid, Option}}
+    end;
+restart_delay_validate([{time_linear_slope, TimeSlope} | Options],
+                       #restart_delay{method = Method} = State)
+    when ((Method =:= undefined) orelse (Method =:= linear)),
+         is_integer(TimeSlope), TimeSlope >= 1 ->
+    restart_delay_validate(Options,
+                           State#restart_delay{
+                               method = linear,
+                               time_slope = TimeSlope});
+restart_delay_validate([{time_linear_min, TimeMin} = Option | Options],
+                       #restart_delay{method = Method,
+                                      time_max = TimeMax} = State)
+    when ((Method =:= undefined) orelse (Method =:= linear)),
+         (is_integer(TimeMin) andalso
+          (TimeMin >= 0) andalso (TimeMin =< ?TIMEOUT_MAX_ERLANG)) orelse
+         (TimeMin =:= limit_min) orelse (TimeMin =:= limit_max) ->
+    NewTimeMin = ?LIMIT_ASSIGN(TimeMin, 0, ?TIMEOUT_MAX_ERLANG),
+    NewTimeMax = if
+        TimeMax =:= undefined ->
+            erlang:max(NewTimeMin, ?RESTART_DELAY_LINEAR_TIME_MAX_DEFAULT);
+        TimeMax =/= undefined ->
+            TimeMax
+    end,
+    if
+        NewTimeMin =< NewTimeMax ->
+            restart_delay_validate(Options,
+                                   State#restart_delay{
+                                       method = linear,
+                                       time_min = NewTimeMin,
+                                       time_max = NewTimeMax});
+        true ->
+            {error, {service_options_restart_delay_invalid, Option}}
+    end;
+restart_delay_validate([{time_linear_max, TimeMax} = Option | Options],
+                       #restart_delay{method = Method,
+                                      time_min = TimeMin} = State)
+    when ((Method =:= undefined) orelse (Method =:= linear)),
+         (is_integer(TimeMax) andalso
+          (TimeMax >= 1) andalso (TimeMax =< ?TIMEOUT_MAX_ERLANG)) orelse
+         (TimeMax =:= limit_min) orelse (TimeMax =:= limit_max) ->
+    NewTimeMax = ?LIMIT_ASSIGN(TimeMax, 1, ?TIMEOUT_MAX_ERLANG),
+    NewTimeMin = if
+        TimeMin =:= undefined ->
+            erlang:min(NewTimeMax, ?RESTART_DELAY_LINEAR_TIME_MIN_DEFAULT);
+        TimeMin =/= undefined ->
+            TimeMin
+    end,
+    if
+        NewTimeMin =< NewTimeMax ->
+            restart_delay_validate(Options,
+                                   State#restart_delay{
+                                       method = linear,
+                                       time_max = NewTimeMax,
+                                       time_min = NewTimeMin});
+        true ->
+            {error, {service_options_restart_delay_invalid, Option}}
+    end;
+restart_delay_validate([{time_absolute, TimeValue} | Options],
+                       #restart_delay{method = Method} = State)
+    when ((Method =:= undefined) orelse (Method =:= absolute)),
+         (is_integer(TimeValue) andalso
+          (TimeValue >= 1) andalso (TimeValue =< ?TIMEOUT_MAX_ERLANG)) orelse
+         (TimeValue =:= limit_min) orelse (TimeValue =:= limit_max) ->
+    NewTimeValue = ?LIMIT_ASSIGN(TimeValue, 1, ?TIMEOUT_MAX_ERLANG),
+    restart_delay_validate(Options,
+                           State#restart_delay{method = absolute,
+                                               time_min = NewTimeValue,
+                                               time_max = NewTimeValue});
+restart_delay_validate([Invalid | _Options],
+                       _State) ->
+    {error, {service_options_restart_delay_invalid, Invalid}}.
+
+restart_delay_value_now(RestartCount,
+                        #restart_delay{method = exponential,
+                                       time_min = TimeMin,
+                                       time_max = TimeMax}) ->
+    TimeValue = (1 bsl RestartCount) * TimeMin,
+    if
+        TimeValue > TimeMax ->
+            TimeMax;
+        true ->
+            TimeValue
+    end;
+restart_delay_value_now(RestartCount,
+                        #restart_delay{method = linear,
+                                       time_min = TimeMin,
+                                       time_max = TimeMax,
+                                       time_slope = TimeSlope}) ->
+    TimeValue = (TimeSlope * RestartCount) + TimeMin,
+    if
+        TimeValue > TimeMax ->
+            TimeMax;
+        true ->
+            TimeValue
+    end;
+restart_delay_value_now(_,
+                        #restart_delay{method = absolute,
+                                       time_min = TimeValue,
+                                       time_max = TimeValue}) ->
+    TimeValue.
+
 count_process_dynamic_validate([],
                                #count_process_dynamic{
                                    method = Method,
@@ -498,12 +758,11 @@ count_process_dynamic_validate([],
                                    rate_max = RateMax,
                                    rate_min = RateMin,
                                    count_process_max = CountMax,
-                                   count_process_min = CountMin} =
-                                       CountProcessDynamic,
+                                   count_process_min = CountMin} = State,
                                CountProcess) ->
     NewMethod = if
         Method =:= undefined ->
-            rate_request;
+            ?COUNT_PROCESS_DYNAMIC_METHOD_DEFAULT;
         Method =/= undefined ->
             Method
     end,
@@ -540,19 +799,27 @@ count_process_dynamic_validate([],
         CountMin =/= undefined ->
             CountMin
     end,
-    {ok,
-     CountProcessDynamic#count_process_dynamic{
-         method = NewMethod,
-         period = NewPeriod,
-         rate_max = NewRateMax,
-         rate_min = NewRateMin,
-         count_process_max = NewCountMax,
-         count_process_min = NewCountMin}};
-count_process_dynamic_validate([{rate_request_max, RateMax} | Options],
+    if
+        (NewMethod =:= rate_request) andalso (NewRateMin == NewRateMax) ->
+            {error, {service_options_count_process_dynamic_invalid,
+                     rate_request_max}};
+        (NewMethod =:= rate_request) andalso (NewCountMin == NewCountMax) ->
+            {error, {service_options_count_process_dynamic_invalid,
+                     count_max}};
+        true ->
+            {ok,
+             State#count_process_dynamic{
+                 method = NewMethod,
+                 period = NewPeriod,
+                 rate_max = NewRateMax,
+                 rate_min = NewRateMin,
+                 count_process_max = NewCountMax,
+                 count_process_min = NewCountMin}}
+    end;
+count_process_dynamic_validate([{rate_request_max, RateMax} = Option | Options],
                                #count_process_dynamic{
                                    method = Method,
-                                   rate_min = RateMin} =
-                                       CountProcessDynamic,
+                                   rate_min = RateMin} = State,
                                CountProcess)
     when ((Method =:= undefined) orelse (Method =:= rate_request)),
          is_number(RateMax), RateMax > 0 ->
@@ -562,16 +829,20 @@ count_process_dynamic_validate([{rate_request_max, RateMax} | Options],
         RateMin =/= undefined ->
             RateMin
     end,
-    count_process_dynamic_validate(Options,
-        CountProcessDynamic#count_process_dynamic{
-            method = rate_request,
-            rate_max = RateMax,
-            rate_min = erlang:min(NewRateMin, RateMax)}, CountProcess);
-count_process_dynamic_validate([{rate_request_min, RateMin} | Options],
+    if
+        NewRateMin =< RateMax ->
+            count_process_dynamic_validate(Options,
+                State#count_process_dynamic{
+                    method = rate_request,
+                    rate_max = RateMax,
+                    rate_min = NewRateMin}, CountProcess);
+        true ->
+            {error, {service_options_count_process_dynamic_invalid, Option}}
+    end;
+count_process_dynamic_validate([{rate_request_min, RateMin} = Option | Options],
                                #count_process_dynamic{
                                    method = Method,
-                                   rate_max = RateMax} =
-                                       CountProcessDynamic,
+                                   rate_max = RateMax} = State,
                                CountProcess)
     when ((Method =:= undefined) orelse (Method =:= rate_request)),
          is_number(RateMin), RateMin > 0 ->
@@ -581,15 +852,19 @@ count_process_dynamic_validate([{rate_request_min, RateMin} | Options],
         RateMax =/= undefined ->
             RateMax
     end,
-    count_process_dynamic_validate(Options,
-        CountProcessDynamic#count_process_dynamic{
-            method = rate_request,
-            rate_max = erlang:max(NewRateMax, RateMin),
-            rate_min = RateMin}, CountProcess);
-count_process_dynamic_validate([{count_max, CountMax} | Options],
+    if
+        RateMin =< NewRateMax ->
+            count_process_dynamic_validate(Options,
+                State#count_process_dynamic{
+                    method = rate_request,
+                    rate_max = NewRateMax,
+                    rate_min = RateMin}, CountProcess);
+        true ->
+            {error, {service_options_count_process_dynamic_invalid, Option}}
+    end;
+count_process_dynamic_validate([{count_max, CountMax} = Option | Options],
                                #count_process_dynamic{
-                                   count_process_min = CountMin} =
-                                       CountProcessDynamic,
+                                   count_process_min = CountMin} = State,
                                CountProcess)
     when (is_float(CountMax) andalso
           (CountMax >= 1.0));
@@ -607,14 +882,18 @@ count_process_dynamic_validate([{count_max, CountMax} | Options],
         CountMin =/= undefined ->
             CountMin
     end,
-    count_process_dynamic_validate(Options,
-        CountProcessDynamic#count_process_dynamic{
-            count_process_max = NewCountMax,
-            count_process_min = NewCountMin}, CountProcess);
-count_process_dynamic_validate([{count_min, CountMin} | Options],
+    if
+        NewCountMin =< NewCountMax ->
+            count_process_dynamic_validate(Options,
+                State#count_process_dynamic{
+                    count_process_max = NewCountMax,
+                    count_process_min = NewCountMin}, CountProcess);
+        true ->
+            {error, {service_options_count_process_dynamic_invalid, Option}}
+    end;
+count_process_dynamic_validate([{count_min, CountMin} = Option | Options],
                                #count_process_dynamic{
-                                   count_process_max = CountMax} =
-                                       CountProcessDynamic,
+                                   count_process_max = CountMax} = State,
                                CountProcess)
     when (is_float(CountMin) andalso
           (CountMin > 0) andalso (CountMin =< 1.0));
@@ -632,26 +911,30 @@ count_process_dynamic_validate([{count_min, CountMin} | Options],
         CountMax =/= undefined ->
             CountMax
     end,
-    count_process_dynamic_validate(Options,
-        CountProcessDynamic#count_process_dynamic{
-            count_process_max = NewCountMax,
-            count_process_min = NewCountMin}, CountProcess);
+    if
+        NewCountMin =< NewCountMax ->
+            count_process_dynamic_validate(Options,
+                State#count_process_dynamic{
+                    count_process_max = NewCountMax,
+                    count_process_min = NewCountMin}, CountProcess);
+        true ->
+            {error, {service_options_count_process_dynamic_invalid, Option}}
+    end;
 count_process_dynamic_validate([{period, Period} | Options],
-                               #count_process_dynamic{} =
-                                   CountProcessDynamic,
+                               #count_process_dynamic{} = State,
                                CountProcess)
     when is_integer(Period), Period > 0,
          Period =< (?TIMEOUT_MAX_ERLANG div 1000) ->
     count_process_dynamic_validate(Options,
-        CountProcessDynamic#count_process_dynamic{
+        State#count_process_dynamic{
             period = Period}, CountProcess);
 count_process_dynamic_validate([Invalid | _Options],
-                               _CountProcessDynamic, _CountProcess) ->
+                               _State, _CountProcess) ->
     {error, {service_options_count_process_dynamic_invalid, Invalid}}.
 
 rate_request_validate([],
                       #rate_request{period = Period,
-                                    rate_max = RateMax} = RateRequest) ->
+                                    rate_max = RateMax} = State) ->
     NewPeriod = if
         Period =:= undefined ->
             ?RATE_REQUEST_PERIOD_DEFAULT;
@@ -664,20 +947,20 @@ rate_request_validate([],
         RateMax =/= undefined ->
             RateMax
     end,
-    {ok, RateRequest#rate_request{period = NewPeriod,
-                                  rate_max = NewRateMax}};
+    {ok, State#rate_request{period = NewPeriod,
+                            rate_max = NewRateMax}};
 rate_request_validate([{value, RateMax} | Options],
-                      #rate_request{} = RateRequest)
+                      #rate_request{} = State)
     when is_number(RateMax), RateMax > 0 ->
     rate_request_validate(Options,
-                          RateRequest#rate_request{rate_max = RateMax});
+                          State#rate_request{rate_max = RateMax});
 rate_request_validate([{period, Period} | Options],
-                      #rate_request{} = RateRequest)
+                      #rate_request{} = State)
     when is_integer(Period), Period > 0,
          Period =< (?TIMEOUT_MAX_ERLANG div 1000) ->
     rate_request_validate(Options,
-                          RateRequest#rate_request{period = Period});
+                          State#rate_request{period = Period});
 rate_request_validate([Invalid | _Options],
-                      _RateRequest) ->
+                      _State) ->
     {error, {service_options_rate_request_max_invalid, Invalid}}.
 
