@@ -9,45 +9,31 @@
 %%% (only log failures).
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2016 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% @copyright 2011-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_services_monitor).
@@ -116,14 +102,13 @@
 
 -record(state,
     {
-        services = cloudi_x_key2value:new() ::
+        services = cloudi_x_key2value:new(maps) ::
             cloudi_x_key2value:
             cloudi_x_key2value(cloudi_service_api:service_id(),
                                pid(), #service{}),
-        changes = dict:new() ::
-            dict_proxy(cloudi_service_api:service_id(),
-                       list({increase | decrease,
-                             number(), number(), number()}))
+        changes = #{} ::
+            #{cloudi_service_api:service_id() :=
+              list({increase | decrease, number(), number(), number()})}
     }).
 
 %%%------------------------------------------------------------------------
@@ -356,7 +341,7 @@ handle_cast({Direction,
     case cloudi_x_key2value:find2(Pid, Services) of
         {ok, {[ServiceId], _}} ->
             Entry = {Direction, RateCurrent, RateLimit, CountProcessLimit},
-            NewChangeList = case dict:find(ServiceId, Changes) of
+            NewChangeList = case maps:find(ServiceId, Changes) of
                 {ok, ChangeList} ->
                     [Entry | ChangeList];
                 error ->
@@ -364,9 +349,9 @@ handle_cast({Direction,
                                       {changes, ServiceId}),
                     [Entry]
             end,
-            {noreply, State#state{changes = dict:store(ServiceId,
-                                                       NewChangeList,
-                                                       Changes)}};
+            {noreply, State#state{changes = maps:put(ServiceId,
+                                                     NewChangeList,
+                                                     Changes)}};
         error ->
             % discard old change
             {noreply, State}
@@ -390,18 +375,18 @@ handle_cast(Request, State) ->
 handle_info({changes, ServiceId},
             #state{services = Services,
                    changes = Changes} = State) ->
-    NewChanges = dict:erase(ServiceId, Changes),
+    NewChanges = maps:remove(ServiceId, Changes),
     case cloudi_x_key2value:find1(ServiceId, Services) of
         {ok, {Pids, #service{count_thread = CountThread}}} ->
-            ChangeList = dict:fetch(ServiceId, Changes),
+            ChangeList = maps:get(ServiceId, Changes),
             CountProcessCurrent = erlang:round(erlang:length(Pids) /
                                                CountThread),
             {I, Rate} = pids_change(ChangeList, CountProcessCurrent),
             NewServices = if
                 I == 0 ->
-                    ?LOG_TRACE("count_process_dynamic: "
+                    ?LOG_TRACE("count_process_dynamic(~p):~n "
                                "constant ~p for ~p requests/second",
-                               [CountProcessCurrent,
+                               [service_id(ServiceId), CountProcessCurrent,
                                 erlang:round(Rate * 10) / 10]),
                     Services;
                 I > 0 ->
@@ -856,9 +841,9 @@ pids_increase_loop(Count, ProcessIndex,
 pids_increase(Count, OldPids, CountProcessCurrent, Rate,
               ServiceId, Services) ->
     ServiceL = service_instance(OldPids, ServiceId, Services),
-    ?LOG_INFO("count_process_dynamic: "
+    ?LOG_INFO("count_process_dynamic(~p):~n "
               "increasing ~p with ~p for ~p requests/second~n~p",
-              [CountProcessCurrent, Count,
+              [service_id(ServiceId), CountProcessCurrent, Count,
                erlang:round(Rate * 10) / 10,
                [P || #service{pids = P} <- ServiceL]]),
     CountProcess = CountProcessCurrent + Count,
@@ -880,9 +865,9 @@ pids_decrease_loop(Count, [#service{pids = Pids} | ServiceL]) ->
 pids_decrease(Count, OldPids, CountProcessCurrent, Rate,
               ServiceId, Services) ->
     ServiceL = service_instance(OldPids, ServiceId, Services),
-    ?LOG_INFO("count_process_dynamic: "
+    ?LOG_INFO("count_process_dynamic(~p):~n "
               "decreasing ~p with ~p for ~p requests/second~n~p",
-              [CountProcessCurrent, Count,
+              [service_id(ServiceId), CountProcessCurrent, Count,
                erlang:round(Rate * 10) / 10,
                [P || #service{pids = P} <- ServiceL]]),
     CountProcess = CountProcessCurrent + Count,
