@@ -6,45 +6,31 @@
 %%% ==CloudI Spawn==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2016 Michael Truog
-%%% @version 1.5.4 {@date} {@time}
+%%% @copyright 2011-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_spawn).
@@ -73,16 +59,10 @@
 -define(ENVIRONMENT_BUFFER_SIZE,   "CLOUDI_API_INIT_BUFFER_SIZE").
 
 -ifdef(CLOUDI_CORE_STANDALONE).
--ifdef(ERLANG_OTP_VERSION_16).
--else.
--ifdef(ERLANG_OTP_VERSION_17).
--else.
 -dialyzer({no_match,
-           [start_external_spawn/18,
+           [start_external_spawn/19,
             update_external/3]}).
 -endif.
--endif.
--endif.
 
 %%%------------------------------------------------------------------------
 %%% External interface
@@ -168,7 +148,7 @@ start_external(ProcessIndex, ProcessCount, ThreadsPerProcess,
                                      DestRefresh, ConfigOptions, ID) of
         {ok,
          SpawnProcess, SpawnProtocol, SocketPath,
-         Rlimits, Owner, Nice, CGroup, Directory,
+         Rlimits, Owner, Nice, CGroup, Chroot, Directory,
          CommandLine, NewFilename, NewArguments, EnvironmentLookup} ->
             {ok, DestDeny, DestAllow} =
                 start_external_threads_params(DestListDeny, DestListAllow),
@@ -193,7 +173,7 @@ start_external(ProcessIndex, ProcessCount, ThreadsPerProcess,
                                          SocketPath,
                                          Pids, Ports,
                                          Rlimits, Owner,
-                                         Nice, CGroup, Directory,
+                                         Nice, CGroup, Chroot, Directory,
                                          ThreadsPerProcess,
                                          CommandLine,
                                          NewFilename,
@@ -222,12 +202,12 @@ update_external(Pids, Ports,
                                      DestRefresh, ConfigOptions, ID) of
         {ok,
          SpawnProcess, SpawnProtocol, SocketPath,
-         Rlimits, Owner, Nice, CGroup, Directory,
+         Rlimits, Owner, Nice, CGroup, Chroot, Directory,
          CommandLine, NewFilename, NewArguments, EnvironmentLookup} ->
             case start_external_spawn(SpawnProcess, SpawnProtocol, SocketPath,
                                       Pids, Ports,
                                       Rlimits, Owner,
-                                      Nice, CGroup, Directory,
+                                      Nice, CGroup, Chroot, Directory,
                                       ThreadsPerProcess,
                                       CommandLine, NewFilename, NewArguments,
                                       Environment, EnvironmentLookup,
@@ -387,6 +367,9 @@ nice(#config_service_options{nice = Nice}) ->
 cgroup(#config_service_options{cgroup = L}, EnvironmentLookup) ->
     cloudi_core_i_os_process:cgroup_format(L, EnvironmentLookup).
 
+chroot(#config_service_options{chroot = Chroot}, EnvironmentLookup) ->
+    cloudi_core_i_os_process:chroot_format(Chroot, EnvironmentLookup).
+
 directory(#config_service_options{directory = Directory}, EnvironmentLookup) ->
     cloudi_core_i_os_process:directory_format(Directory, EnvironmentLookup).
 
@@ -439,7 +422,7 @@ start_external_spawn_params(ProcessIndex, ProcessCount, ThreadsPerProcess,
     EnvironmentLookup = environment_lookup(),
     case start_external_spawn_params_parse(Filename, Arguments, ConfigOptions,
                                            EnvironmentLookup) of
-        {ok, CommandLine, NewFilename, NewArguments, Directory} ->
+        {ok, CommandLine, NewFilename, NewArguments, Chroot, Directory} ->
             case cloudi_x_supool:get(cloudi_core_i_os_spawn) of
                 SpawnProcess when is_pid(SpawnProcess) ->
                     SpawnProtocol = if
@@ -456,7 +439,7 @@ start_external_spawn_params(ProcessIndex, ProcessCount, ThreadsPerProcess,
                     CGroup = cgroup(ConfigOptions, EnvironmentLookup),
                     {ok,
                      SpawnProcess, SpawnProtocol, SocketPath,
-                     Rlimits, Owner, Nice, CGroup, Directory,
+                     Rlimits, Owner, Nice, CGroup, Chroot, Directory,
                      CommandLine, NewFilename, NewArguments,
                      EnvironmentLookup};
                 undefined ->
@@ -489,13 +472,19 @@ start_external_spawn_params_parse(Filename, Arguments, ConfigOptions,
                 {ok, NewArguments} ->
                     CommandLine = [NewFilename |
                                    string:tokens(NewArguments, [0])],
-                    case directory(ConfigOptions, EnvironmentLookup) of
-                        {ok, Directory} ->
-                            {ok,
-                             CommandLine,
-                             NewFilename,
-                             NewArguments,
-                             Directory};
+                    case chroot(ConfigOptions, EnvironmentLookup) of
+                        {ok, Chroot} ->
+                            case directory(ConfigOptions, EnvironmentLookup) of
+                                {ok, Directory} ->
+                                    {ok,
+                                     CommandLine,
+                                     NewFilename,
+                                     NewArguments,
+                                     Chroot,
+                                     Directory};
+                                {error, _} = Error ->
+                                    Error
+                            end;
                         {error, _} = Error ->
                             Error
                     end;
@@ -551,7 +540,8 @@ start_external_threads(ThreadsPerProcess,
                           ConfigOptions, ID).
 
 start_external_spawn(SpawnProcess, SpawnProtocol, SocketPath,
-                     Pids, Ports, Rlimits, Owner, Nice, CGroup, Directory,
+                     Pids, Ports, Rlimits, Owner,
+                     Nice, CGroup, Chroot, Directory,
                      ThreadsPerProcess, CommandLine,
                      Filename, Arguments, Environment,
                      EnvironmentLookup, Protocol, BufferSize) ->
@@ -569,6 +559,7 @@ start_external_spawn(SpawnProcess, SpawnProtocol, SocketPath,
                                       GroupI,
                                       string_terminate(GroupStr),
                                       Nice,
+                                      string_terminate(Chroot),
                                       string_terminate(Directory),
                                       string_terminate(Filename),
                                       string_terminate(Arguments),
