@@ -30,7 +30,7 @@
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
 %%% @copyright 2014-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @version 1.7.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_http_client).
@@ -855,7 +855,7 @@ client_debug_log(fatal, Message, Args) ->
 client_debug_start(off) ->
     undefined;
 client_debug_start(_) ->
-    cloudi_x_uuid:get_v1_time(os).
+    cloudi_timestamp:microseconds_monotonic().
 
 client_debug_end(off, _, _, _, _, _, _, _) ->
     undefined;
@@ -869,7 +869,7 @@ client_debug_end(Level, HttpCode, Method,
                      "headers_out(~p)~n"
                      "request_out(~p)",
                      [HttpCode, Method,
-                      (cloudi_x_uuid:get_v1_time(os) -
+                      (cloudi_timestamp:microseconds_monotonic() -
                        RequestStartMicroSec) / 1000.0,
                       HeadersIncoming, Request,
                       HeadersOutgoing, Response]).
