@@ -30,7 +30,7 @@
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
 %%% @copyright 2012-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @version 1.7.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_http_cowboy_handler).
@@ -942,18 +942,18 @@ get_query_string_external_text([{K, V} | L]) ->
     end.
 
 request_time_start() ->
-    cloudi_x_uuid:get_v1_time(os).
+    cloudi_timestamp:microseconds_monotonic().
 
 request_time_end_success(HttpCode, Method, NameIncoming, NameOutgoing,
                          RequestStartMicroSec) ->
     ?LOG_TRACE("~w ~s ~s (to ~s) ~p ms",
                [HttpCode, Method, NameIncoming, NameOutgoing,
-                (cloudi_x_uuid:get_v1_time(os) -
+                (cloudi_timestamp:microseconds_monotonic() -
                  RequestStartMicroSec) / 1000.0]).
 
 request_time_end_error(HttpCode, Method, NameIncoming, NameOutgoing,
                        RequestStartMicroSec, Reason) ->
-    RequestTime = (cloudi_x_uuid:get_v1_time(os) -
+    RequestTime = (cloudi_timestamp:microseconds_monotonic() -
                    RequestStartMicroSec) / 1000.0,
     if
         NameOutgoing =:= undefined ->
@@ -967,20 +967,20 @@ request_time_end_error(HttpCode, Method, NameIncoming, NameOutgoing,
     end.
 
 websocket_time_start() ->
-    cloudi_x_uuid:get_v1_time(os).
+    cloudi_timestamp:microseconds_monotonic().
 
 websocket_time_end_success(NameIncoming, NameOutgoing,
                            RequestStartMicroSec) ->
     ?LOG_TRACE("~s (to ~s) ~p ms",
                [NameIncoming, NameOutgoing,
-                (cloudi_x_uuid:get_v1_time(os) -
+                (cloudi_timestamp:microseconds_monotonic() -
                  RequestStartMicroSec) / 1000.0]).
 
 websocket_time_end_error(NameIncoming, NameOutgoing,
                          RequestStartMicroSec, Reason) ->
     ?LOG_WARN("~s (to ~s) ~p ms: ~p",
               [NameIncoming, NameOutgoing,
-               (cloudi_x_uuid:get_v1_time(os) -
+               (cloudi_timestamp:microseconds_monotonic() -
                 RequestStartMicroSec) / 1000.0, Reason]).
 
 websocket_request_end(Name, NewTimeout, OldTimeout) ->
