@@ -30,7 +30,7 @@
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
 %%% @copyright 2013-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @version 1.7.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_http_elli_handler).
@@ -249,6 +249,8 @@ handle(Req,
 %% create ETS tables or start supervised processes in a supervisor
 %% tree.
 handle_event(elli_startup, [], _) -> ok;
+%% elli_reconfigure is an event after elli:set_callback/3 is called
+handle_event(elli_reconfigure, [], _) -> ok;
 %% request_complete fires *after* Elli has sent the response to the
 %% client. Timings contains timestamps of events like when the
 %% connection was accepted, when request parsing finished, when the
@@ -398,18 +400,18 @@ get_query_string_external_text([{K, V} | L]) ->
     end.
 
 request_time_start() ->
-    cloudi_x_uuid:get_v1_time(os).
+    cloudi_timestamp:microseconds_monotonic().
 
 request_time_end_success(HttpCode, Method, NameIncoming, NameOutgoing,
                          RequestStartMicroSec) ->
     ?LOG_TRACE("~w ~s ~s (to ~s) ~p ms",
                [HttpCode, Method, NameIncoming, NameOutgoing,
-                (cloudi_x_uuid:get_v1_time(os) -
+                (cloudi_timestamp:microseconds_monotonic() -
                  RequestStartMicroSec) / 1000.0]).
 
 request_time_end_error(HttpCode, Method, NameIncoming, NameOutgoing,
                        RequestStartMicroSec, Reason) ->
-    RequestTime = (cloudi_x_uuid:get_v1_time(os) -
+    RequestTime = (cloudi_timestamp:microseconds_monotonic() -
                    RequestStartMicroSec) / 1000.0,
     if
         NameOutgoing =:= undefined ->
