@@ -9,45 +9,31 @@
 %%% arguments.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2015-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2015-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_http_rest).
@@ -107,7 +93,7 @@
 
 -type initialize_f() :: fun((Args :: list(),
                              Timeout :: cloudi_service_api:
-                                        timeout_milliseconds(),
+                                        timeout_initialize_value_milliseconds(),
                              Dispatcher :: cloudi:dispatcher()) ->
     {ok, State :: any()} |
     {stop, Reason :: any()} |
@@ -139,7 +125,8 @@
                       {noreply, NewState :: any()} |
                       {stop, Reason :: any(), NewState :: any()}).
 -type terminate_f() :: fun((Reason :: any(),
-                            Timeout :: cloudi:timeout_value_milliseconds(),
+                            Timeout :: cloudi_service_api:
+                                       timeout_terminate_value_milliseconds(),
                             State :: any()) ->
                            ok).
 -export_type([method/0,
@@ -158,19 +145,13 @@
         arity :: 11
     }).
 
--ifdef(ERLANG_OTP_VERSION_16).
--type dict_proxy(_Key, _Value) :: dict().
--else.
--type dict_proxy(Key, Value) :: dict:dict(Key, Value).
--endif.
-
 -record(state,
     {
         prefix :: cloudi:service_name_pattern(),
         lookup :: cloudi_x_trie:cloudi_x_trie(),
         info_f :: info_f() | undefined,
         terminate_f :: terminate_f() | undefined,
-        content_types :: dict_proxy(atom(), {request | attachment, binary()}),
+        content_types :: #{atom() := {request | attachment, binary()}},
         content_disposition :: boolean(),
         debug_level :: off | trace | debug | info | warn | error | fatal,
         api_state :: any()
@@ -242,10 +223,10 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
         FormatN
     end, Formats0),
     ContentTypeLookupN = lists:foldl(fun(FormatN, ContentTypeLookup0) ->
-        dict:store(erlang:list_to_atom(FormatN),
-                   cloudi_x_trie:fetch("." ++ FormatN, ContentTypes),
-                   ContentTypeLookup0)
-    end, dict:new(), FormatsN),
+        maps:put(erlang:list_to_atom(FormatN),
+                 cloudi_x_trie:fetch("." ++ FormatN, ContentTypes),
+                 ContentTypeLookup0)
+    end, #{}, FormatsN),
     MethodListsN = cloudi_x_trie:to_list(lists:foldr(fun({Method, Path, _},
                                                          MethodLists0) ->
         MethodString = if
@@ -491,7 +472,7 @@ subscribe_paths(Method, Path, Formats, API, Lookup, Dispatcher)
 response_info_headers(ResponseInfo0, Name, Format,
                       ContentTypes, ContentDisposition)
     when is_list(ResponseInfo0) ->
-    {AttachmentGuess, ContentType} = dict:fetch(Format, ContentTypes),
+    {AttachmentGuess, ContentType} = maps:get(Format, ContentTypes),
     ResponseInfo1 = if
         (ContentDisposition =:= true) andalso
         (AttachmentGuess =:= attachment) ->
