@@ -6,45 +6,31 @@
 %%% ==CloudI Monitoring Metrics==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2015-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @copyright 2015-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_monitoring_cloudi).
@@ -98,7 +84,7 @@
 
 -record(service_data,
     {
-        process_info :: dict_proxy(pid(), #process_info{}),
+        process_info :: #{pid() := #process_info{}},
         % modifications to ?ETS_PID2METRIC
         ets_insert = [] :: list(pid_object()),
         ets_delete = [] :: list(pid()),
@@ -107,24 +93,13 @@
         count_external = 0 :: non_neg_integer(),
         concurrency_internal = 0 :: non_neg_integer(),
         concurrency_external = 0 :: non_neg_integer(),
-        scopes = dict:new() :: dict_proxy(atom(), #scope_data{}),
+        scopes = #{} :: #{atom() := #scope_data{}},
         metrics = [] :: metric_list()
     }).
 
 % cloudi_core_i_service_internal and cloudi_core_i_service_external
 % use maps when available/stable
--ifdef(ERLANG_OTP_VERSION_16).
--else.
--ifdef(ERLANG_OTP_VERSION_17).
--else.
--define(ERLANG_OTP_VERSION_18_FEATURES, true).
--endif.
--endif.
--ifdef(ERLANG_OTP_VERSION_18_FEATURES).
 -define(MAP_SIZE(M),         maps:size(M)).
--else.
--define(MAP_SIZE(M),         dict:size(M)).
--endif.
 
 %%%------------------------------------------------------------------------
 %%% External interface functions
@@ -285,58 +260,58 @@ services_update(undefined, ServicesNew, ProcessInfo0, QueuedEmptySize,
                          #service_data{process_info = ProcessInfo1,
                                        ets_insert = Inserts0,
                                        metrics = Metrics0} = Changes1) ->
-            ServiceMetricId = service_metric_id_from_service(Service,
-                                                             EnvironmentLookup),
-            {Inserts3,
-             ProcessInfo3} = lists:foldl(fun(PidNew, {Inserts1, ProcessInfo2}) ->
-                Inserts2 = if
-                    UseAspectsOnly =:= true ->
-                        Inserts1;
-                    UseAspectsOnly =:= false ->
-                        [{PidNew, MetricPrefix ++ ServiceMetricId, Driver} |
-                         Inserts1]
-                end,
-                {Inserts2, process_info_store(PidNew, ProcessInfo2)}
-            end, {Inserts0, ProcessInfo1}, PidsNew),
-            {Metrics1,
-             ProcessInfo4} = service_metrics(PidsNew, ProcessInfo3, Service,
-                                             QueuedEmptySize,
-                                             ServicesNew, ServiceMetricId),
-            services_accumulate(Service,
-                                Changes1#service_data{process_info = ProcessInfo4,
-                                                      ets_insert = Inserts3,
-                                                      metrics = Metrics1 ++
-                                                                Metrics0})
-        end, #service_data{process_info = ProcessInfo0}, ServicesNew),
-        #service_data{process_info = ProcessInfoN,
-                      ets_insert = InsertsN,
-                      count_internal = CountInternal,
-                      count_external = CountExternal,
-                      concurrency_internal = ConcurrencyInternal,
-                      concurrency_external = ConcurrencyExternal,
-                      scopes = Scopes,
-                      metrics = MetricsN} = ChangesN,
-        if
-            UseAspectsOnly =:= true ->
-                ok;
-            UseAspectsOnly =:= false ->
-                true = ets:delete_all_objects(?ETS_PID2METRIC),
-                true = ets:insert(?ETS_PID2METRIC, InsertsN)
-        end,
-        {services_metrics(CountInternal, CountExternal,
-                          ConcurrencyInternal, ConcurrencyExternal,
-                          Scopes) ++ MetricsN,
-         ProcessInfoN};
-    services_update(ServicesOld, ServicesNew, ProcessInfo0, QueuedEmptySize,
-                    MetricPrefix, UseAspectsOnly, Driver, EnvironmentLookup) ->
-        ChangesN = cloudi_x_key2value:
-                   fold1(fun(ID, PidsNew,
-                             #service{} = Service,
-                             #service_data{process_info = ProcessInfo1,
-                                           ets_insert = Inserts0,
-                                           ets_delete = Deletes0,
-                                           metrics = Metrics0} = Changes1) ->
-            ServiceMetricId = service_metric_id_from_service(Service,
+        ServiceMetricId = service_metric_id_from_service(Service,
+                                                         EnvironmentLookup),
+        {Inserts3,
+         ProcessInfo3} = lists:foldl(fun(PidNew, {Inserts1, ProcessInfo2}) ->
+            Inserts2 = if
+                UseAspectsOnly =:= true ->
+                    Inserts1;
+                UseAspectsOnly =:= false ->
+                    [{PidNew, MetricPrefix ++ ServiceMetricId, Driver} |
+                     Inserts1]
+            end,
+            {Inserts2, process_info_store(PidNew, ProcessInfo2)}
+        end, {Inserts0, ProcessInfo1}, PidsNew),
+        {Metrics1,
+         ProcessInfo4} = service_metrics(PidsNew, ProcessInfo3, Service,
+                                         QueuedEmptySize,
+                                         ServicesNew, ServiceMetricId),
+        services_accumulate(Service,
+                            Changes1#service_data{process_info = ProcessInfo4,
+                                                  ets_insert = Inserts3,
+                                                  metrics = Metrics1 ++
+                                                            Metrics0})
+    end, #service_data{process_info = ProcessInfo0}, ServicesNew),
+    #service_data{process_info = ProcessInfoN,
+                  ets_insert = InsertsN,
+                  count_internal = CountInternal,
+                  count_external = CountExternal,
+                  concurrency_internal = ConcurrencyInternal,
+                  concurrency_external = ConcurrencyExternal,
+                  scopes = Scopes,
+                  metrics = MetricsN} = ChangesN,
+    if
+        UseAspectsOnly =:= true ->
+            ok;
+        UseAspectsOnly =:= false ->
+            true = ets:delete_all_objects(?ETS_PID2METRIC),
+            true = ets:insert(?ETS_PID2METRIC, InsertsN)
+    end,
+    {services_metrics(CountInternal, CountExternal,
+                      ConcurrencyInternal, ConcurrencyExternal,
+                      Scopes) ++ MetricsN,
+     ProcessInfoN};
+services_update(ServicesOld, ServicesNew, ProcessInfo0, QueuedEmptySize,
+                MetricPrefix, UseAspectsOnly, Driver, EnvironmentLookup) ->
+    ChangesN = cloudi_x_key2value:
+               fold1(fun(ID, PidsNew,
+                         #service{} = Service,
+                         #service_data{process_info = ProcessInfo1,
+                                       ets_insert = Inserts0,
+                                       ets_delete = Deletes0,
+                                       metrics = Metrics0} = Changes1) ->
+        ServiceMetricId = service_metric_id_from_service(Service,
                                                          EnvironmentLookup),
         Changes2 = case cloudi_x_key2value:find1(ID, ServicesOld) of
             {ok, {PidsNew, #service{}}} ->
@@ -441,7 +416,7 @@ nodes_update(NodesVisible, NodesHidden, NodesAll) ->
 %%%------------------------------------------------------------------------
 
 scopes_accumulate_internal(Scope, Concurrency, Scopes) ->
-    dict:update(Scope,
+    maps:update_with(Scope,
                 fun(#scope_data{count_internal = Count,
                                 concurrency_internal = ConcurrencySum} = V) ->
         V#scope_data{count_internal = Count + 1,
@@ -450,7 +425,7 @@ scopes_accumulate_internal(Scope, Concurrency, Scopes) ->
                      concurrency_internal = Concurrency}, Scopes).
 
 scopes_accumulate_external(Scope, Concurrency, Scopes) ->
-    dict:update(Scope,
+    maps:update_with(Scope,
                 fun(#scope_data{count_external = Count,
                                 concurrency_external = ConcurrencySum} = V) ->
         V#scope_data{count_external = Count + 1,
@@ -498,9 +473,29 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
              Memory,
              Messages,
              ReductionsNow,
+             RequestPidInfo,
+             InfoPidInfo,
              ProcessInfoN} = case erlang:tuple_size(State) of
                 30 -> % duo_mode == false
                     state = erlang:element(1, State),
+                    RequestPid = erlang:element(23, State),
+                    InfoPid = erlang:element(24, State),
+                    {RequestPidInfoValue,
+                     ProcessInfo1} = if
+                        RequestPid =:= undefined ->
+                            {{undefined, undefined, undefined},
+                             ProcessInfo0};
+                        is_pid(RequestPid) ->
+                            process_info_update(RequestPid, ProcessInfo0)
+                    end,
+                    {InfoPidInfoValue,
+                     ProcessInfo2} = if
+                        InfoPid =:= undefined ->
+                            {{undefined, undefined, undefined},
+                             ProcessInfo1};
+                        is_pid(InfoPid) ->
+                            process_info_update(InfoPid, ProcessInfo1)
+                    end,
                     {?MAP_SIZE(erlang:element(3, State)),  % send_timeouts
                      erlang:element(9, State),             % queued
                      erlang:element(10, State),            % queued_size
@@ -509,10 +504,24 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                      ServiceMemory,
                      ServiceMessages,
                      ServiceReductionsNow,
-                     ProcessInfo0};
+                     RequestPidInfoValue,
+                     InfoPidInfoValue,
+                     ProcessInfo2};
                 15 -> % duo_mode == true
                     state_duo = erlang:element(1, State),
                     Dispatcher =  erlang:element(13, State),
+                    RequestPid = erlang:element(14, State),
+                    {RequestPidInfoValue,
+                     ProcessInfo1} = if
+                        RequestPid =:= undefined ->
+                            {{undefined, undefined, undefined},
+                             ProcessInfo0};
+                        is_pid(RequestPid) ->
+                            process_info_update(RequestPid, ProcessInfo0)
+                    end,
+                    InfoPidInfoValue = {ServiceMemory,
+                                        ServiceMessages,
+                                        ServiceReductionsNow},
                     DispatcherOutgoing = case service_state(Dispatcher) of
                         {ok, DispatcherState} -> % gen_server/proc_lib
                             30 = erlang:tuple_size(DispatcherState),
@@ -524,29 +533,29 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                     {MemoryValue,
                      MessagesValue,
                      ReductionsNowValue,
-                     ProcessInfo2} = case process_info_update(Dispatcher,
-                                                              ProcessInfo0) of
-                        {{undefined, _, _}, ProcessInfo1} ->
+                     ProcessInfo3} = case process_info_update(Dispatcher,
+                                                              ProcessInfo1) of
+                        {{undefined, _, _}, ProcessInfo2} ->
                             {ServiceMemory,
                              ServiceMessages,
                              ServiceReductionsNow,
-                             ProcessInfo1};
+                             ProcessInfo2};
                         {{DispatcherMemory,
                           DispatcherMessages,
-                          DispatcherReductionsNow}, ProcessInfo1}
+                          DispatcherReductionsNow}, ProcessInfo2}
                         when ServiceReductionsNow =:= undefined;
                              DispatcherReductionsNow =:= undefined ->
                             {ServiceMemory + DispatcherMemory,
                              ServiceMessages + DispatcherMessages,
                              undefined,
-                             ProcessInfo1};
+                             ProcessInfo2};
                         {{DispatcherMemory,
                           DispatcherMessages,
-                          DispatcherReductionsNow}, ProcessInfo1} ->
+                          DispatcherReductionsNow}, ProcessInfo2} ->
                             {ServiceMemory + DispatcherMemory,
                              ServiceMessages + DispatcherMessages,
                              ServiceReductionsNow + DispatcherReductionsNow,
-                             ProcessInfo1}
+                             ProcessInfo2}
                     end,
                     {DispatcherOutgoing,
                      erlang:element(6, State),   % queued
@@ -556,7 +565,9 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                      MemoryValue,
                      MessagesValue,
                      ReductionsNowValue,
-                     ProcessInfo2}
+                     RequestPidInfoValue,
+                     InfoPidInfoValue,
+                     ProcessInfo3}
             end,
             QueuedRequestsLength = cloudi_x_pqueue4:len(QueuedRequests),
             QueuedRequestsSizeN = if
@@ -568,19 +579,66 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                     QueuedRequestsSize0
             end,
             QueuedInfoLength = queue:len(QueuedInfo),
-            Metrics0 = if
+            Metrics0 = [],
+            Metrics1 = case RequestPidInfo of
+                {undefined, _, _} ->
+                    Metrics0;
+                {RequestPidMemory,
+                 RequestPidMessages,
+                 RequestPidReductionsNow} ->
+                    % the metrics here will only appear with
+                    % the service configuration option request_pid_uses > 1
+                    % with the metrics update becoming more
+                    % likely with higher values
+                    [metric(gauge, MetricPrefix ++ [request, memory],
+                            RequestPidMemory),
+                     metric(gauge, MetricPrefix ++ [request, message_queue_len],
+                            RequestPidMessages) |
+                     if
+                        RequestPidReductionsNow =:= undefined ->
+                            Metrics0;
+                        is_integer(RequestPidReductionsNow) ->
+                            [metric(spiral,
+                                    MetricPrefix ++ [request, reductions],
+                                    RequestPidReductionsNow) | Metrics0]
+                     end]
+            end,
+            Metrics2 = case InfoPidInfo of
+                {undefined, _, _} ->
+                    Metrics1;
+                {InfoPidMemory,
+                 InfoPidMessages,
+                 InfoPidReductionsNow} ->
+                    % the metrics here will only appear with
+                    % the service configuration option info_pid_uses > 1
+                    % with the metrics update becoming more
+                    % likely with higher values
+                    [metric(gauge, MetricPrefix ++ [info, memory],
+                            InfoPidMemory),
+                     metric(gauge, MetricPrefix ++ [info, message_queue_len],
+                            InfoPidMessages) |
+                     if
+                        InfoPidReductionsNow =:= undefined ->
+                            Metrics1;
+                        is_integer(InfoPidReductionsNow) ->
+                            [metric(spiral,
+                                    MetricPrefix ++ [info, reductions],
+                                    InfoPidReductionsNow) | Metrics1]
+                     end]
+            end,
+            Metrics3 = if
                 Outgoing =:= undefined ->
-                    [];
+                    Metrics2;
                 is_integer(Outgoing) ->
                     [metric(gauge, MetricPrefix ++ [outgoing_requests],
-                            Outgoing)]
+                            Outgoing) | Metrics2]
             end,
             MetricsN = if
                 ReductionsNow =:= undefined ->
-                    Metrics0;
+                    Metrics3;
                 is_integer(ReductionsNow) ->
                     [metric(spiral, MetricPrefix ++ [reductions],
-                            ReductionsNow) | Metrics0]
+                            ReductionsNow) | Metrics3]
             end,
             {[metric(gauge, MetricPrefix ++ [memory],
                      Memory),
@@ -600,7 +658,7 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                         external, ProcessInfo0, Pid,
                         QueuedEmptySize, MetricPrefix) ->
     case service_state(Pid) of
-        {ok, {_, State}} -> % gen_fsm
+        {ok, {_, State}} -> % gen_statem
             40 = erlang:tuple_size(State),
             state = erlang:element(1, State),
             Outgoing = ?MAP_SIZE(erlang:element(3, State)),  % send_timeouts
@@ -666,7 +724,7 @@ service_metrics_pid_external([Pid | Pids], Metrics, ProcessInfo0,
     {[_],
      #service{process_index = ProcessIndex}} =
         cloudi_x_key2value:fetch2(Pid, Services),
-    ThreadIndex = case dict:find(ProcessIndex, ThreadIndexLookup) of
+    ThreadIndex = case maps:find(ProcessIndex, ThreadIndexLookup) of
         {ok, ThreadIndexNext} ->
             ThreadIndexNext;
         error ->
@@ -682,8 +740,8 @@ service_metrics_pid_external([Pid | Pids], Metrics, ProcessInfo0,
                                              QueuedEmptySize,
                                              ThreadMetricPrefix),
     service_metrics_pid_external(Pids, MetricsNew ++ Metrics, ProcessInfoN,
-                                 dict:store(ProcessIndex,
-                                            ThreadIndex + 1, ThreadIndexLookup),
+                                 maps:put(ProcessIndex,
+                                          ThreadIndex + 1, ThreadIndexLookup),
                                  QueuedEmptySize,
                                  Services, MetricPrefix).
 
@@ -693,7 +751,7 @@ service_metrics_pid(internal, Pids, ProcessInfo,
                                  QueuedEmptySize, Services, MetricPrefix);
 service_metrics_pid(external, Pids, ProcessInfo,
                     QueuedEmptySize, Services, MetricPrefix) ->
-    service_metrics_pid_external(Pids, [], ProcessInfo, dict:new(),
+    service_metrics_pid_external(Pids, [], ProcessInfo, #{},
                                  QueuedEmptySize, Services, MetricPrefix).
 
 service_metrics(Pids, ProcessInfo0,
@@ -720,7 +778,7 @@ service_state(Pid) ->
 
 services_metrics(CountInternal, CountExternal,
                  ConcurrencyInternal, ConcurrencyExternal, Scopes) ->
-    dict:fold(fun(Scope, 
+    maps:fold(fun(Scope, 
                   #scope_data{
                       count_internal = ScopeCountInternal,
                       count_external = ScopeCountExternal,
@@ -742,7 +800,7 @@ services_metrics(CountInternal, CountExternal,
                 ScopeCountExternal) | ScopeMetrics]
     end,
     [metric(gauge, [scopes, count],
-            dict:size(Scopes)),
+            maps:size(Scopes)),
      metric(gauge, [concurrency],
             ConcurrencyInternal + ConcurrencyExternal),
      metric(gauge, [count],
@@ -762,7 +820,7 @@ process_info_store(Pid, ProcessInfo) ->
         [{memory, MemoryNew},
          {message_queue_len, MessagesNew},
          {reductions, ReductionsNew}] ->
-            case dict:find(Pid, ProcessInfo) of
+            case maps:find(Pid, ProcessInfo) of
                 {ok, #process_info{reductions = ReductionsOld}} ->
                     ReductionsNow = if
                         ReductionsOld =:= undefined ->
@@ -774,15 +832,15 @@ process_info_store(Pid, ProcessInfo) ->
                                             message_queue_len = MessagesNew,
                                             reductions = ReductionsNew,
                                             reductions_now = ReductionsNow},
-                    dict:store(Pid, InfoNew, ProcessInfo);
+                    maps:put(Pid, InfoNew, ProcessInfo);
                 error ->
                     InfoNew = #process_info{memory = MemoryNew,
                                             message_queue_len = MessagesNew,
                                             reductions = ReductionsNew},
-                    dict:store(Pid, InfoNew, ProcessInfo)
+                    maps:put(Pid, InfoNew, ProcessInfo)
             end;
         undefined ->
-            dict:store(Pid, #process_info{}, ProcessInfo)
+            maps:put(Pid, #process_info{}, ProcessInfo)
     end.
 
 process_info_update(Pid, ProcessInfo) ->
@@ -790,7 +848,7 @@ process_info_update(Pid, ProcessInfo) ->
         [{memory, MemoryNew},
          {message_queue_len, MessagesNew},
          {reductions, ReductionsNew}] ->
-            case dict:find(Pid, ProcessInfo) of
+            case maps:find(Pid, ProcessInfo) of
                 {ok, #process_info{reductions = ReductionsOld}} ->
                     ReductionsNow = if
                         ReductionsOld =:= undefined ->
@@ -803,21 +861,21 @@ process_info_update(Pid, ProcessInfo) ->
                                             reductions = ReductionsNew,
                                             reductions_now = ReductionsNow},
                     {{MemoryNew, MessagesNew, ReductionsNow},
-                     dict:store(Pid, InfoNew, ProcessInfo)};
+                     maps:put(Pid, InfoNew, ProcessInfo)};
                 error ->
                     InfoNew = #process_info{memory = MemoryNew,
                                             message_queue_len = MessagesNew,
                                             reductions = ReductionsNew},
                     {{MemoryNew, MessagesNew, undefined},
-                     dict:store(Pid, InfoNew, ProcessInfo)}
+                     maps:put(Pid, InfoNew, ProcessInfo)}
             end;
         undefined ->
             {{undefined, undefined, undefined},
-             dict:store(Pid, #process_info{}, ProcessInfo)}
+             maps:put(Pid, #process_info{}, ProcessInfo)}
     end.
 
 process_info_find(Pid, ProcessInfo) ->
-    case dict:find(Pid, ProcessInfo) of
+    case maps:find(Pid, ProcessInfo) of
         {ok, #process_info{memory = Memory,
                            message_queue_len = Messages,
                            reductions_now = ReductionsNow}} ->
@@ -827,7 +885,7 @@ process_info_find(Pid, ProcessInfo) ->
     end.
 
 process_info_erase(Pid, ProcessInfo) ->
-    dict:erase(Pid, ProcessInfo).
+    maps:remove(Pid, ProcessInfo).
 
 process_info_metrics({Memory, Messages, ReductionsNow}, MetricPrefix) ->
     L0 = [],
