@@ -8,45 +8,31 @@
 %%% No disk index is maintained, but an in-memory index is kept.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2014-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2016 Michael Truog
-%%% @version 1.5.4 {@date} {@time}
+%%% @copyright 2014-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_write_ahead_logging).
@@ -79,17 +65,12 @@
         retries = 0 :: non_neg_integer()
     }).
 
--ifdef(ERLANG_OTP_VERSION_16).
--type dict_proxy(_Key, _Value) :: dict().
--else.
--type dict_proxy(Key, Value) :: dict:dict(Key, Value).
--endif.
 -record(state,
     {
         file = undefined :: undefined | string(),
         compression = undefined :: undefined | 0..9, % zlib compression level
         position = undefined :: undefined | non_neg_integer(),
-        chunks = dict:new() :: dict_proxy(cloudi_service:trans_id(), #chunk{}),
+        chunks = #{} :: #{cloudi_service:trans_id() := #chunk{}},
         chunks_free = [] :: list(#chunk{}) % ordered
     }).
 
@@ -120,12 +101,12 @@
 erase(ChunkId,
       #state{file = FilePath,
              chunks = Chunks} = State) ->
-    Chunk = dict:fetch(ChunkId, Chunks),
+    {Chunk, NewChunks} = maps:take(ChunkId, Chunks),
     #chunk{request = ChunkRequest} = Chunk,
     {ok, Fd} = file_open_tmp(FilePath),
     NewState = erase_chunk(Chunk, Fd, State),
     ok = file_close_tmp(FilePath, Fd),
-    {ChunkRequest, NewState#state{chunks = dict:erase(ChunkId, Chunks)}}.
+    {ChunkRequest, NewState#state{chunks = NewChunks}}.
 
 -spec erase_retry(ChunkId :: cloudi_service:trans_id(),
                   RetryMax :: non_neg_integer(),
@@ -136,7 +117,7 @@ erase(ChunkId,
 erase_retry(ChunkId, RetryMax, RetryF,
             #state{file = FilePath,
                    chunks = Chunks} = State) ->
-    Chunk = dict:fetch(ChunkId, Chunks),
+    {Chunk, NewChunks} = maps:take(ChunkId, Chunks),
     #chunk{request = ChunkRequest,
            retries = Retries} = Chunk,
     NewChunkId = case RetryF(ChunkRequest, Retries < RetryMax) of
@@ -150,24 +131,23 @@ erase_retry(ChunkId, RetryMax, RetryF,
             {ok, Fd} = file_open_tmp(FilePath),
             NewState = erase_chunk(Chunk, Fd, State),
             ok = file_close_tmp(FilePath, Fd),
-            NewState#state{chunks = dict:erase(ChunkId, Chunks)};
+            NewState#state{chunks = NewChunks};
         is_binary(NewChunkId) ->
             NewChunk = Chunk#chunk{retries = Retries + 1},
-            NewChunks = dict:erase(ChunkId, Chunks),
-            State#state{chunks = dict:store(NewChunkId, NewChunk, NewChunks)}
+            State#state{chunks = maps:put(NewChunkId, NewChunk, NewChunks)}
     end.
 
 -spec fetch_keys(State :: #state{}) ->
     list(cloudi_service:trans_id()).
 
 fetch_keys(#state{chunks = Chunks}) ->
-    lists:sort(dict:fetch_keys(Chunks)). % oldest -> newest
+    lists:sort(maps:keys(Chunks)). % oldest -> newest
 
 -spec size(State :: #state{}) ->
     non_neg_integer().
 
 size(#state{chunks = Chunks}) ->
-    dict:size(Chunks).
+    maps:size(Chunks).
 
 -spec size_free(State :: #state{}) ->
     non_neg_integer().
@@ -182,7 +162,7 @@ size_free(#state{chunks_free = ChunksFree}) ->
 
 store_end(ChunkId, Chunk,
           #state{chunks = Chunks} = State) ->
-    State#state{chunks = dict:store(ChunkId, Chunk, Chunks)}.
+    State#state{chunks = maps:put(ChunkId, Chunk, Chunks)}.
 
 -spec store_fail(Chunk :: #chunk{},
                  State :: #state{}) ->
@@ -260,7 +240,7 @@ update(ChunkId, UpdateF,
               position = Position,
               chunks = Chunks,
               chunks_free = ChunksFree} = State) ->
-    Chunk = dict:fetch(ChunkId, Chunks),
+    Chunk = maps:get(ChunkId, Chunks),
     #chunk{request = ChunkRequest} = Chunk,
     case UpdateF(ChunkRequest) of
         undefined ->
@@ -280,7 +260,7 @@ update(ChunkId, UpdateF,
                     NewChunk = #chunk{size = NewChunkSize,
                                       position = Position,
                                       request = NewChunkRequest},
-                    NewChunks = dict:store(NewChunkId, NewChunk, Chunks),
+                    NewChunks = maps:put(NewChunkId, NewChunk, Chunks),
                     State#state{chunks = NewChunks,
                                 position = NewPosition};
                 {#chunk{size = ChunkSize,
@@ -288,7 +268,7 @@ update(ChunkId, UpdateF,
                     chunk_write(ChunkSize, NewChunkSizeUsed,
                                 NewChunkData, ChunkPosition, Fd),
                     NewChunk = ChunkFree#chunk{request = NewChunkRequest},
-                    NewChunks = dict:store(NewChunkId, NewChunk, Chunks),
+                    NewChunks = maps:put(NewChunkId, NewChunk, Chunks),
                     State#state{chunks = NewChunks,
                                 chunks_free = NewChunksFree}
             end,
@@ -329,25 +309,25 @@ file_close_tmp(FilePath, Fd) ->
 
 chunk_write(ChunkSize, ChunkSizeUsed, ChunkData, Position, Fd) ->
     ChunkSizeZero = (ChunkSize - ChunkSizeUsed),
-    {ok, _} = file:position(Fd, Position),
-    ok = file:write(Fd, <<ChunkSize:64/unsigned-integer-big,
-                          ChunkSizeUsed:64/unsigned-integer-big,
-                          ChunkData/binary,
-                          0:(ChunkSizeZero * 8)>>),
+    ok = file:pwrite(Fd, Position,
+                     <<ChunkSize:64/unsigned-integer-big,
+                       ChunkSizeUsed:64/unsigned-integer-big,
+                       ChunkData/binary,
+                       0:(ChunkSizeZero * 8)>>),
     Position + ?CHUNK_OVERHEAD + ChunkSize.
 
 chunk_erase_last(ChunkSize, Position, Fd) ->
-    {ok, _} = file:position(Fd, Position),
-    ok = file:write(Fd, <<0:64,
-                          0:64,
-                          0:(ChunkSize * 8)>>),
+    ok = file:pwrite(Fd, Position,
+                     <<0:64,
+                       0:64,
+                       0:(ChunkSize * 8)>>),
     ok.
 
 chunk_free(ChunkSize, Position, Fd) ->
-    {ok, _} = file:position(Fd, Position),
-    ok = file:write(Fd, <<ChunkSize:64/unsigned-integer-big,
-                          0:64,
-                          0:(ChunkSize * 8)>>),
+    ok = file:pwrite(Fd, Position,
+                     <<ChunkSize:64/unsigned-integer-big,
+                       0:64,
+                       0:(ChunkSize * 8)>>),
     ok.
 
 erase_chunk(#chunk{size = ChunkSize,
@@ -402,7 +382,7 @@ chunk_recover_used(Position, ChunkSize, ChunkSizeUsed,
                                    position = Position,
                                    request = ChunkRequest},
                     chunks_recover(NewPosition,
-                                   dict:store(ChunkId, Chunk, Chunks),
+                                   maps:put(ChunkId, Chunk, Chunks),
                                    ChunksFree, Fd, RetryF);
                 {ok, ChunkId} ->
                     ChunkEnd = (ChunkSize - ChunkSizeUsed),
@@ -412,7 +392,7 @@ chunk_recover_used(Position, ChunkSize, ChunkSizeUsed,
                                            position = Position,
                                            request = ChunkRequest},
                             chunks_recover(NewPosition,
-                                           dict:store(ChunkId, Chunk, Chunks),
+                                           maps:put(ChunkId, Chunk, Chunks),
                                            ChunksFree, Fd, RetryF);
                         {error, Reason} ->
                             {error, {chunk_corrupt, Reason}}
