@@ -5,45 +5,31 @@
 %%% ==CloudI Service Validate Tests==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2015-2017, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
 %%% @copyright 2015-2017 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_validate_SUITE).
@@ -190,10 +176,12 @@ init_per_testcase(TestCase, Config)
         % using proplist configuration format, not the tuple/record format
         [{prefix, ?SERVICE_PREFIX2},
          {module, ?MODULE},
-         {args, [{mode, incr_source}]}],
+         {args, [{mode, incr_source}]},
+         {options, [{automatic_loading, false}]}],
         [{prefix, ?SERVICE_PREFIX2},
          {module, ?MODULE},
-         {args, [{mode, incr_dest}]}],
+         {args, [{mode, incr_dest}]},
+         {options, [{automatic_loading, false}]}],
         [{prefix, ?SERVICE_PREFIX1},
          {module, cloudi_service_validate},
          {args,
@@ -216,10 +204,12 @@ init_per_testcase(TestCase, Config)
         % using proplist configuration format, not the tuple/record format
         [{prefix, ?SERVICE_PREFIX2},
          {module, ?MODULE},
-         {args, [{mode, incr_source}]}],
+         {args, [{mode, incr_source}]},
+         {options, [{automatic_loading, false}]}],
         [{prefix, ?SERVICE_PREFIX2},
          {module, ?MODULE},
-         {args, [{mode, incr_dest}]}],
+         {args, [{mode, incr_dest}]},
+         {options, [{automatic_loading, false}]}],
         [{prefix, ?SERVICE_PREFIX1},
          {module, cloudi_service_validate},
          {args,
@@ -242,10 +232,12 @@ init_per_testcase(TestCase, Config)
         % using proplist configuration format, not the tuple/record format
         [{prefix, ?SERVICE_PREFIX2},
          {module, ?MODULE},
-         {args, [{mode, incr_source}]}],
+         {args, [{mode, incr_source}]},
+         {options, [{automatic_loading, false}]}],
         [{prefix, ?SERVICE_PREFIX2},
          {module, ?MODULE},
-         {args, [{mode, incr_dest}]}],
+         {args, [{mode, incr_dest}]},
+         {options, [{automatic_loading, false}]}],
         [{prefix, ?SERVICE_PREFIX1},
          {module, cloudi_service_validate},
          {args,
@@ -268,10 +260,12 @@ init_per_testcase(TestCase, Config)
         % using proplist configuration format, not the tuple/record format
         [{prefix, ?SERVICE_PREFIX2},
          {module, ?MODULE},
-         {args, [{mode, incr_source}]}],
+         {args, [{mode, incr_source}]},
+         {options, [{automatic_loading, false}]}],
         [{prefix, ?SERVICE_PREFIX2},
          {module, ?MODULE},
-         {args, [{mode, incr_dest}]}],
+         {args, [{mode, incr_dest}]},
+         {options, [{automatic_loading, false}]}],
         [{prefix, ?SERVICE_PREFIX1},
          {module, cloudi_service_validate},
          {args,
