@@ -11,45 +11,31 @@
 %%% can be avoided.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
+%%%
+%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2016 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% @copyright 2011-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cpg_data).
@@ -269,7 +255,7 @@ get_local_members(GroupName, Groups) ->
             % to keep return values consistent with pg2
             {error, {'no_such_group', GroupName}};
         {ok, Pattern, #cpg_data{local = Local}} ->
-            {ok, Pattern, [Pid || #cpg_data_pid{pid = Pid} <- Local]}
+            {ok, Pattern, Local}
     end.
 
 %%-------------------------------------------------------------------------
@@ -295,8 +281,7 @@ get_local_members(GroupName, Exclude, Groups)
             % to keep return values consistent with pg2
             {error, {'no_such_group', GroupName}};
         {ok, Pattern, #cpg_data{local = Local}} ->
-            Members = [Pid || #cpg_data_pid{pid = Pid} <- Local,
-                       Pid =/= Exclude],
+            Members = [Pid || Pid <- Local, Pid =/= Exclude],
             if
                 Members == [] ->
                     % to keep return values consistent with pg2
@@ -326,7 +311,7 @@ get_remote_members(GroupName, Groups) ->
             % to keep return values consistent with pg2
             {error, {'no_such_group', GroupName}};
         {ok, Pattern, #cpg_data{remote = Remote}} ->
-            {ok, Pattern, [Pid || #cpg_data_pid{pid = Pid} <- Remote]}
+            {ok, Pattern, Remote}
     end.
 
 %%-------------------------------------------------------------------------
@@ -352,8 +337,7 @@ get_remote_members(GroupName, Exclude, Groups)
             % to keep return values consistent with pg2
             {error, {'no_such_group', GroupName}};
         {ok, Pattern, #cpg_data{remote = Remote}} ->
-            Members = [Pid || #cpg_data_pid{pid = Pid} <- Remote,
-                       Pid =/= Exclude],
+            Members = [Pid || Pid <- Remote, Pid =/= Exclude],
             if
                 Members == [] ->
                     % to keep return values consistent with pg2
@@ -930,11 +914,11 @@ group_find(GroupName, {DictI, GroupsData}) ->
             error
     end.
 
-pick(1, [#cpg_data_pid{pid = Pid}], Pattern) ->
+pick(1, [Pid], Pattern) ->
     {ok, Pattern, Pid};
 
 pick(N, L, Pattern) ->
-    #cpg_data_pid{pid = Pid} = lists:nth(random(N), L),
+    Pid = lists:nth(random(N), L),
     {ok, Pattern, Pid}.
 
 pick_i(I, I, _, [], [], _, GroupName, _) ->
@@ -947,21 +931,18 @@ pick_i(I, I, Length, Filtered, [], _, _, Pattern) ->
     {ok, Pattern, lists:nth((I rem Length) + 1, Filtered)};
 
 pick_i(I, I, Length, Filtered,
-       [#cpg_data_pid{pid = Exclude} | L],
-       Exclude, GroupName, Pattern) ->
+       [Exclude | L], Exclude, GroupName, Pattern) ->
     pick_i(I, I, Length, Filtered, L, Exclude, GroupName, Pattern);
 
-pick_i(I, I, _, _, [#cpg_data_pid{pid = Pid} | _], _, _, Pattern) ->
+pick_i(I, I, _, _, [Pid | _], _, _, Pattern) ->
     {ok, Pattern, Pid};
 
 pick_i(I, Random, Length, Filtered,
-       [#cpg_data_pid{pid = Exclude} | L],
-       Exclude, GroupName, Pattern) ->
+       [Exclude | L], Exclude, GroupName, Pattern) ->
     pick_i(I + 1, Random, Length, Filtered, L, Exclude, GroupName, Pattern);
 
 pick_i(I, Random, Length, Filtered,
-       [#cpg_data_pid{pid = Pid} | L],
-       Exclude, GroupName, Pattern) ->
+       [Pid | L], Exclude, GroupName, Pattern) ->
     pick_i(I + 1, Random, Length + 1, [Pid | Filtered],
            L, Exclude, GroupName, Pattern).
 
